@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: dvi_interpreter.c 745 2010-05-03 18:06:33Z dcf21 $
+// $Id: dvi_interpreter.c 911 2010-07-28 19:44:44Z rpc25 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -256,7 +256,7 @@ int dviInOpChar(dviInterpreterState *interp, DVIOperator *op)
   s = interp->currentString+strlen(interp->currentString); // s now points to the \0
   if      (charToTypeset == 40) snprintf(s, 3, "%s", "\\(");
   else if (charToTypeset == 41) snprintf(s, 3, "%s", "\\)");
-  else                          snprintf(s, 2, "%s", (char *)&charToTypeset);
+  else                          snprintf(s, 2, "%c", charToTypeset);
   return 0;
  }
 
@@ -627,11 +627,9 @@ int dviInOpPostPost(dviInterpreterState *interp, DVIOperator *op)
 // Accumulate characters output in special mode into a string
 int dviSpecialChar(dviInterpreterState *interp, DVIOperator *op)
  {
-  int c;
   char *s;
-  c = op->op;
   s = interp->spString+strlen(interp->spString);
-  snprintf(s, SSTR_LENGTH, "%s", (char *)&c);
+  snprintf(s, SSTR_LENGTH, "%c", op->op);
   return 0;
  }
 
@@ -1023,6 +1021,7 @@ int dviTypeset(dviInterpreterState *interp)
   width  = 0.0;
   height = 0.0;
   depth  = 0.0;
+  italic = 0.0;
   while (*s != '\0')
    {
     if ((err=dviGetCharSize(interp, (unsigned char) *s, size))!=0) return err;
