@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: eps_piechart.c 763 2010-05-13 16:37:14Z dcf21 $
+// $Id: eps_piechart.c 875 2010-07-08 15:44:28Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -56,6 +56,7 @@ void eps_pie_ReadAccessibleData(EPSComm *x)
   char             errbuffer[LSTR_LENGTH];
   char            *LabelString;
   List            *UsingList, *EveryList;
+  unsigned char    AutoUsingList=0;
   Dict            *tempdict;
   with_words       ww_default;
   double          *ordinate_raster, acc;
@@ -69,8 +70,9 @@ void eps_pie_ReadAccessibleData(EPSComm *x)
   if (x->current->plotdata == NULL) { ppl_error(ERR_MEMORY, -1, -1,"Out of memory"); *(x->status) = 1; return; }
 
   // Merge together with words to form a final set
-  eps_withwords_default(&ww_default, &x->current->settings, 1, 0, 0, settings_term_current.colour==SW_ONOFF_ON);
+  eps_withwords_default(&ww_default, &x->current->settings, 1, 0, 0, 0, settings_term_current.colour==SW_ONOFF_ON);
   with_words_merge(&pd->ww_final, &pd->ww, &x->current->settings.FuncStyle, &ww_default, NULL, NULL, 1);
+  pd->ww_final.linespoints = SW_STYLE_LINES; // In case FuncStyle is something bonkers like contourplot
 
   // Make raster on which to evaluate parametric functions
   ordinate_raster = (double *)lt_malloc(x->current->settings.samples * sizeof(double));
@@ -88,15 +90,15 @@ void eps_pie_ReadAccessibleData(EPSComm *x)
   ErrCount = DATAFILE_NERRS;
   NExpect  = 1;
 
-  if (eps_plot_AddUsingItemsForWithWords(&pd->ww_final, &NExpect, UsingList)) { *(x->status) = 1; return; } // Add extra using items for, e.g. "linewidth $3".
+  if (eps_plot_AddUsingItemsForWithWords(&pd->ww_final, &NExpect, &AutoUsingList, UsingList)) { *(x->status) = 1; return; } // Add extra using items for, e.g. "linewidth $3".
 
   if (pd->function == 0) // Read data from file
    {
     if (DEBUG) { sprintf(temp_err_string, "Reading data from file '%s' for piechart item %d", pd->filename, x->current->id); ppl_log(temp_err_string); }
-    DataFile_read(x->current->plotdata, &status, errbuffer, pd->filename, pd->index, pd->UsingRowCols, UsingList, EveryList, LabelString, NExpect, pd->SelectCriterion, pd->continuity, NULL, -1, &ErrCount);
+    DataFile_read(x->current->plotdata, &status, errbuffer, pd->filename, pd->index, pd->UsingRowCols, UsingList, AutoUsingList, EveryList, LabelString, NExpect, pd->SelectCriterion, pd->continuity, NULL, -1, &ErrCount);
    } else {
     if (DEBUG) { sprintf(temp_err_string, "Reading data from functions for piechart item %d", x->current->id); ppl_log(temp_err_string); }
-    DataFile_FromFunctions(ordinate_raster, 1, x->current->settings.samples, &settings_graph_current.Tmin, x->current->plotdata, &status, errbuffer, pd->functions, pd->NFunctions, UsingList, LabelString, NExpect, pd->SelectCriterion, pd->continuity, NULL, -1, &ErrCount);
+    DataFile_FromFunctions(ordinate_raster, 1, x->current->settings.samples, &settings_graph_current.Tmin, NULL, 0, NULL, x->current->plotdata, &status, errbuffer, pd->functions, pd->NFunctions, UsingList, AutoUsingList, LabelString, NExpect, pd->SelectCriterion, pd->continuity, NULL, -1, &ErrCount);
    }
   if (status) { ppl_error(ERR_GENERAL, -1, -1, errbuffer); x->current->plotdata[0]=NULL; }
 
@@ -326,12 +328,11 @@ void eps_pie_RenderEPS(EPSComm *x)
      {
       for (j=0; j<blk->BlockPosition; j++)
        {
+        size = fabs(blk->data_real[0 + Ncolumns*j].d) / pd->PieChart_total * 2 * M_PI; // Angular size of wedge
         if ((blk->text[j]!=NULL)&&(blk->text[j][0]!='\0'))
          {
           int    ArrowType;
-          double a;
-          size             = fabs(blk->data_real[0 + Ncolumns*j].d) / pd->PieChart_total * 2 * M_PI; // Angular size of wedge
-          a                = angle+size/2; // Central angle of wedge
+          double a = angle+size/2; // Central angle of wedge
 
           // Test different radii where label can be placed
           best_label_rpos   = 0.7;
@@ -368,7 +369,7 @@ void eps_pie_RenderEPS(EPSComm *x)
            { // Labelling pie wedges inside pie
             int pageno = x->LaTeXpageno++;
             eps_core_SetColour(x, &ww_txt, 1);
-            canvas_EPSRenderTextItem(x, pageno, (xpos+rad*sin(a)*best_label_rpos)/M_TO_PS, (ypos+rad*cos(a)*best_label_rpos)/M_TO_PS, SW_HALIGN_CENT, SW_VALIGN_CENT, x->CurrentColour, x->current->settings.FontSize, 0.0, NULL, NULL);
+            canvas_EPSRenderTextItem(x, NULL, pageno, (xpos+rad*sin(a)*best_label_rpos)/M_TO_PS, (ypos+rad*cos(a)*best_label_rpos)/M_TO_PS, SW_HALIGN_CENT, SW_VALIGN_CENT, x->CurrentColour, x->current->settings.FontSize, 0.0, NULL, NULL);
            }
           else if (ArrowType==SW_PIEKEYPOS_OUTSIDE)
            { // Labelling pie wedges around edge of pie
@@ -390,7 +391,7 @@ void eps_pie_RenderEPS(EPSComm *x)
             eps_core_SetColour(x, &ww, 1);
             IF_NOT_INVISIBLE fprintf(x->epsbuffer, "newpath\n%.2f %.2f moveto\n%.2f %.2f lineto\nclosepath\nstroke\n",xpos+rad*sin(a),ypos+rad*cos(a),xpos+rad*sin(a)*1.05,ypos+rad*cos(a)*1.05);
             eps_core_SetColour(x, &ww_txt, 1);
-            canvas_EPSRenderTextItem(x, pageno, (xpos+rad*sin(a)*1.08)/M_TO_PS, (ypos+rad*cos(a)*1.08)/M_TO_PS, hal, val, x->CurrentColour, x->current->settings.FontSize, 0.0, NULL, NULL);
+            canvas_EPSRenderTextItem(x, NULL, pageno, (xpos+rad*sin(a)*1.08)/M_TO_PS, (ypos+rad*cos(a)*1.08)/M_TO_PS, hal, val, x->CurrentColour, x->current->settings.FontSize, 0.0, NULL, NULL);
            }
           else // Labelling pie wedges in a key
            {
@@ -433,7 +434,7 @@ void eps_pie_RenderEPS(EPSComm *x)
 
             // Write text next to icon
             eps_core_SetColour(x, &ww_txt, 1);
-            canvas_EPSRenderTextItem(x, pageno, (h+st)/M_TO_PS, v/M_TO_PS, SW_HALIGN_LEFT, SW_VALIGN_CENT, x->CurrentColour, x->current->settings.FontSize, 0.0, NULL, NULL);
+            canvas_EPSRenderTextItem(x, NULL, pageno, (h+st)/M_TO_PS, v/M_TO_PS, SW_HALIGN_LEFT, SW_VALIGN_CENT, x->CurrentColour, x->current->settings.FontSize, 0.0, NULL, NULL);
             key_vpos += TextHeight[ItemNo];
            }
          }
@@ -450,7 +451,7 @@ END_LABEL_PRINTING:
     if ((x->current->settings.title != NULL) && (x->current->settings.title[0] != '\0'))
      {
       int pageno = x->LaTeXpageno++;
-      canvas_EPSRenderTextItem(x, pageno, xpos/M_TO_PS, (vtitle+8)/M_TO_PS, SW_HALIGN_CENT, SW_VALIGN_BOT, x->CurrentColour, x->current->settings.FontSize, 0.0, NULL, NULL);
+      canvas_EPSRenderTextItem(x, NULL, pageno, xpos/M_TO_PS, (vtitle+8)/M_TO_PS, SW_HALIGN_CENT, SW_VALIGN_BOT, x->CurrentColour, x->current->settings.FontSize, 0.0, NULL, NULL);
      }
    }
 
