@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: eps_plot.h 632 2010-02-27 00:22:39Z dcf21 $
+// $Id: eps_plot.h 866 2010-07-03 15:40:57Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -28,7 +28,7 @@
 
 #include "eps_comm.h"
 
-int eps_plot_AddUsingItemsForWithWords(with_words *ww, int *NExpect, List *UsingList);
+int eps_plot_AddUsingItemsForWithWords(with_words *ww, int *NExpect, unsigned char *AutoUsingList, List *UsingList);
 
 void eps_plot_ReadAccessibleData(EPSComm *x);
 void eps_plot_SampleFunctions(EPSComm *x);
@@ -37,7 +37,7 @@ void eps_plot_YieldUpText(EPSComm *x);
 void eps_plot_RenderEPS(EPSComm *x);
 
 void eps_plot_WithWordsFromUsingItems(with_words *ww, double *DataRow, int Ncolumns);
-int  eps_plot_WithWordsCheckUsingItemsDimLess(with_words *ww, value *FirstValues, int Ncolumns);
+int  eps_plot_WithWordsCheckUsingItemsDimLess(with_words *ww, value *FirstValues, int Ncolumns, int *NDataCols);
 
 #endif
 
