@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: eps_plot_axespaint.h 810 2010-05-24 18:03:38Z dcf21 $
+// $Id: eps_plot_axespaint.h 874 2010-07-08 12:31:56Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -22,9 +22,12 @@
 #ifndef _PPL_EPS_PLOT_AXESPAINT_H
 #define _PPL_EPS_PLOT_AXESPAINT_H 1
 
+#include "ppl_settings.h"
+
 #include "eps_comm.h"
 
-void eps_plot_axespaint(EPSComm *x, double origin_x, double origin_y, double width, double height, double zdepth);
+void eps_plot_axispaint(EPSComm *x, with_words *ww, settings_axis *a, const int xyz, const double CP2, const unsigned char Lr, const double x1, const double y1, const double *z1, const double x2, const double y2, const double *z2, const double theta_a, const double theta_b, double *OutputWidth, const unsigned char PrintLabels);
+void eps_plot_axespaint(EPSComm *x, double origin_x, double origin_y, double width, double height, double zdepth, int pass);
 
 #endif
 
