@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: eps_plot_filledregion.c 745 2010-05-03 18:06:33Z dcf21 $
+// $Id: eps_plot_filledregion.c 852 2010-06-27 00:31:20Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -180,7 +180,7 @@ static void OutputPath(FilledRegionHandle *fr, FilledRegionAxisCrossing *CrossPo
  {
   unsigned char face, sense, inside_ahead, FillSide, CurrentFace, first_point=1, first_subpath=1, fail=0;
   int i, inew, j, k, l, dir_x, dir_y;
-  ListItem *li, *lil;
+  ListItem *li, *lil=NULL;
   FilledRegionPoint *p;
 
   // Clear used flags
