@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: eps_plot_legend.c 814 2010-05-28 15:14:41Z dcf21 $
+// $Id: eps_plot_legend.c 905 2010-07-25 19:31:16Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -46,9 +46,48 @@
 #include "eps_colours.h"
 #include "eps_comm.h"
 #include "eps_core.h"
+#include "eps_plot_canvas.h"
 #include "eps_plot_legend.h"
 #include "eps_plot_styles.h"
 
+// Private routines for sorting 3D positions by depth and azimuth when working out where to put a legend on a 3D plot
+
+static int SortByDepth(const void *x, const void *y)
+ {
+  const double *xd = (const double *)x;
+  const double *yd = (const double *)y;
+  if (xd[2]>yd[2]) return  1;
+  if (xd[2]<yd[2]) return -1;
+  return 0;
+ }
+
+static double SortByAzimuthXCentre, SortByAzimuthYCentre, SortByAzimuthTarget;
+
+static int SortByAzimuthProximity(const void *x, const void *y)
+ {
+  const double *xd = (const double *)x;
+  const double *yd = (const double *)y;
+  double ax,ay;
+  ax = fabs(atan2(xd[0]-SortByAzimuthXCentre , xd[1]-SortByAzimuthYCentre) - SortByAzimuthTarget);
+  ay = fabs(atan2(yd[0]-SortByAzimuthXCentre , yd[1]-SortByAzimuthYCentre) - SortByAzimuthTarget);
+  while (ax>2*M_PI) ax-=2*M_PI; if (ax>M_PI) ax = 2*M_PI-ax;
+  while (ay>2*M_PI) ay-=2*M_PI; if (ay>M_PI) ay = 2*M_PI-ay;
+  if (ax>ay) return  1;
+  if (ax<ay) return -1;
+  return 0;
+ }
+
+static void TriSwap(double *a, double *b)
+ {
+  double t[3];
+  memcpy(t,a,3*sizeof(double));
+  memcpy(a,b,3*sizeof(double));
+  memcpy(b,t,3*sizeof(double));
+  return;
+ }
+
+// Graph legend API routines
+
 void GraphLegend_YieldUpText(EPSComm *x)
  {
   canvas_plotdesc *pd;
@@ -90,9 +129,9 @@ void GraphLegend_YieldUpText(EPSComm *x)
         if (pd->axis1set || pd->axis2set || pd->axis3set) // Print axes to use
          {
           strcpy(cptr+k, " axes "); k+=strlen(cptr+k);
-          if (pd->axis1set) { sprintf(cptr+k, "$%c%d$", "xyz"[pd->axis1xyz], pd->axis1); k+=strlen(cptr+k); }
-          if (pd->axis2set) { sprintf(cptr+k, "$%c%d$", "xyz"[pd->axis2xyz], pd->axis2); k+=strlen(cptr+k); }
-          if (pd->axis3set) { sprintf(cptr+k, "$%c%d$", "xyz"[pd->axis3xyz], pd->axis3); k+=strlen(cptr+k); }
+          if (pd->axis1set) { sprintf(cptr+k, "$%c%d$", "xyzc"[pd->axis1xyz], pd->axis1); k+=strlen(cptr+k); }
+          if (pd->axis2set) { sprintf(cptr+k, "$%c%d$", "xyzc"[pd->axis2xyz], pd->axis2); k+=strlen(cptr+k); }
+          if (pd->axis3set) { sprintf(cptr+k, "$%c%d$", "xyzc"[pd->axis3xyz], pd->axis3); k+=strlen(cptr+k); }
          }
         if (pd->EverySet>0) { sprintf(cptr+k, " every $%d$", pd->EveryList[0]); k+=strlen(cptr+k); } // Print out 'every' clause of plot command
         if (pd->EverySet>1) { sprintf(cptr+k, ":$%d$", pd->EveryList[1]); k+=strlen(cptr+k); }
@@ -162,7 +201,7 @@ void GraphLegend_ArrangeToHeight(EPSComm *x, double TrialHeight, double *Attaine
 void GraphLegend_Render(EPSComm *x, double width, double height, double zdepth)
  {
   double fs=x->current->settings.FontSize, CombinedHeight=0.0, MinimumHeight=0.0;
-  double xoff, yoff;
+  double xoff=0, yoff=0;
   double ColumnX[MAX_LEGEND_COLUMNS], ColumnHeight[MAX_LEGEND_COLUMNS];
   double BestHeight, AttainedHeight, TrialHeight;
   int    NDataSet, Ncolumns, ColumnNItems[MAX_LEGEND_COLUMNS];
@@ -263,31 +302,84 @@ void GraphLegend_Render(EPSComm *x, double width, double height, double zdepth)
 
   // Translate legend to desired place on canvas (2D case)
   if (!x->current->ThreeDim)
-   switch (kp)
-    {
-     case SW_KEYPOS_TR:      xoff = width   - ColumnX[Ncolumns]   - LEGEND_MARGIN; yoff = height                     - LEGEND_MARGIN; break;
-     case SW_KEYPOS_TM:      xoff = width/2 - ColumnX[Ncolumns]/2                ; yoff = height                     - LEGEND_MARGIN; break;
-     case SW_KEYPOS_TL:      xoff =                                 LEGEND_MARGIN; yoff = height                     - LEGEND_MARGIN; break;
-     case SW_KEYPOS_MR:      xoff = width   - ColumnX[Ncolumns]   - LEGEND_MARGIN; yoff = height/2 + AttainedHeight/2               ; break;
-     case SW_KEYPOS_MM:      xoff = width/2 - ColumnX[Ncolumns]/2                ; yoff = height/2 + AttainedHeight/2               ; break;
-     case SW_KEYPOS_ML:      xoff =                                 LEGEND_MARGIN; yoff = height/2 + AttainedHeight/2               ; break;
-     case SW_KEYPOS_BR:      xoff = width   - ColumnX[Ncolumns]   - LEGEND_MARGIN; yoff =            AttainedHeight  + LEGEND_MARGIN; break;
-     case SW_KEYPOS_BM:      xoff = width/2 - ColumnX[Ncolumns]/2                ; yoff =            AttainedHeight  + LEGEND_MARGIN; break;
-     case SW_KEYPOS_BL:      xoff =                                 LEGEND_MARGIN; yoff =            AttainedHeight  + LEGEND_MARGIN; break;
-     case SW_KEYPOS_ABOVE:   xoff = width/2 - ColumnX[Ncolumns]/2                ; yoff =            AttainedHeight  + LEGEND_MARGIN + x->current->PlotTopMargin    - x->current->settings.OriginY.real*M_TO_PS; break;
-     case SW_KEYPOS_BELOW:   xoff = width/2 - ColumnX[Ncolumns]/2                ; yoff =                            - LEGEND_MARGIN + x->current->PlotBottomMargin - x->current->settings.OriginY.real*M_TO_PS; break;
-     case SW_KEYPOS_OUTSIDE: xoff =                                 LEGEND_MARGIN; yoff = height                     - LEGEND_MARGIN;
-                             xoff+= x->current->PlotRightMargin - x->current->settings.OriginX.real*M_TO_PS; break;
-    }
+   {
+    switch (kp)
+     {
+      case SW_KEYPOS_TR:      xoff = width   - ColumnX[Ncolumns]   - LEGEND_MARGIN; yoff = height                     - LEGEND_MARGIN; break;
+      case SW_KEYPOS_TM:      xoff = width/2 - ColumnX[Ncolumns]/2                ; yoff = height                     - LEGEND_MARGIN; break;
+      case SW_KEYPOS_TL:      xoff =                                 LEGEND_MARGIN; yoff = height                     - LEGEND_MARGIN; break;
+      case SW_KEYPOS_MR:      xoff = width   - ColumnX[Ncolumns]   - LEGEND_MARGIN; yoff = height/2 + AttainedHeight/2               ; break;
+      case SW_KEYPOS_MM:      xoff = width/2 - ColumnX[Ncolumns]/2                ; yoff = height/2 + AttainedHeight/2               ; break;
+      case SW_KEYPOS_ML:      xoff =                                 LEGEND_MARGIN; yoff = height/2 + AttainedHeight/2               ; break;
+      case SW_KEYPOS_BR:      xoff = width   - ColumnX[Ncolumns]   - LEGEND_MARGIN; yoff =            AttainedHeight  + LEGEND_MARGIN; break;
+      case SW_KEYPOS_BM:      xoff = width/2 - ColumnX[Ncolumns]/2                ; yoff =            AttainedHeight  + LEGEND_MARGIN; break;
+      case SW_KEYPOS_BL:      xoff =                                 LEGEND_MARGIN; yoff =            AttainedHeight  + LEGEND_MARGIN; break;
+      case SW_KEYPOS_ABOVE:   xoff = width/2 - ColumnX[Ncolumns]/2                ; yoff =            AttainedHeight  + LEGEND_MARGIN + x->current->PlotTopMargin    - x->current->settings.OriginY.real*M_TO_PS; break;
+      case SW_KEYPOS_BELOW:   xoff = width/2 - ColumnX[Ncolumns]/2                ; yoff =                            - LEGEND_MARGIN + x->current->PlotBottomMargin - x->current->settings.OriginY.real*M_TO_PS; break;
+      case SW_KEYPOS_OUTSIDE: xoff =                                 LEGEND_MARGIN; yoff = height                     - LEGEND_MARGIN;
+                              xoff+= x->current->PlotRightMargin - x->current->settings.OriginX.real*M_TO_PS; break;
+     }
+    xoff += x->current->settings.OriginX.real * M_TO_PS;
+    yoff += x->current->settings.OriginY.real * M_TO_PS;
+   }
 
   // Translate legend to desired place on canvas (3D case)
   else
    {
-    xoff = yoff = 0.0;
-   }
+    SortByAzimuthTarget = -999;
 
-  xoff += (x->current->settings.OriginX.real + x->current->settings.KeyXOff.real) * M_TO_PS;
-  yoff += (x->current->settings.OriginY.real + x->current->settings.KeyYOff.real) * M_TO_PS;
+    switch (kp)
+     {
+      case SW_KEYPOS_TR: SortByAzimuthTarget =  1*M_PI/4; xoff =                       LEGEND_MARGIN + x->current->PlotRightMargin; yoff = AttainedHeight  + LEGEND_MARGIN; break;
+      case SW_KEYPOS_TM: SortByAzimuthTarget =  0*M_PI/4; xoff = -ColumnX[Ncolumns]/2                                             ; yoff = AttainedHeight + LEGEND_MARGIN + x->current->PlotTopMargin; break;
+      case SW_KEYPOS_TL: SortByAzimuthTarget = -1*M_PI/4; xoff = -ColumnX[Ncolumns]  - LEGEND_MARGIN + x->current->PlotLeftMargin ; yoff = AttainedHeight  + LEGEND_MARGIN; break;
+      case SW_KEYPOS_MR: SortByAzimuthTarget =  2*M_PI/4; xoff =                       LEGEND_MARGIN + x->current->PlotRightMargin; yoff = AttainedHeight/2               ; break;
+      case SW_KEYPOS_MM:                                  xoff = -ColumnX[Ncolumns]/2 + x->current->settings.OriginX.real * M_TO_PS; yoff = AttainedHeight/2 + x->current->settings.OriginY.real * M_TO_PS; break;
+      case SW_KEYPOS_ML: SortByAzimuthTarget = -2*M_PI/4; xoff = -ColumnX[Ncolumns]  - LEGEND_MARGIN + x->current->PlotLeftMargin ; yoff = AttainedHeight/2               ; break;
+      case SW_KEYPOS_BR: SortByAzimuthTarget =  3*M_PI/4; xoff =                       LEGEND_MARGIN + x->current->PlotRightMargin; yoff =                 - LEGEND_MARGIN; break;
+      case SW_KEYPOS_BM: SortByAzimuthTarget =  4*M_PI/4; xoff = -ColumnX[Ncolumns]/2                                             ; yoff = - LEGEND_MARGIN + x->current->PlotBottomMargin; break;
+      case SW_KEYPOS_BL: SortByAzimuthTarget = -3*M_PI/4; xoff = -ColumnX[Ncolumns]  - LEGEND_MARGIN + x->current->PlotLeftMargin ; yoff =                 - LEGEND_MARGIN; break;
+
+      case SW_KEYPOS_ABOVE:   xoff = -ColumnX[Ncolumns]/2 + x->current->settings.OriginX.real * M_TO_PS;
+                              yoff = AttainedHeight + LEGEND_MARGIN + x->current->PlotTopMargin;
+                              break;
+      case SW_KEYPOS_BELOW:   xoff = -ColumnX[Ncolumns]/2 + x->current->settings.OriginX.real * M_TO_PS;
+                              yoff =                - LEGEND_MARGIN + x->current->PlotBottomMargin;
+                              break;
+      case SW_KEYPOS_OUTSIDE: xoff =  LEGEND_MARGIN + x->current->PlotRightMargin;
+                              yoff = -LEGEND_MARGIN + x->current->PlotTopMargin;
+                              break;
+     }
+
+    // Find the vertex of the graph closest to the target azimuth
+    if (SortByAzimuthTarget>-100)
+     {
+      int i;
+      double xap, yap, zap, data[3*8];
+      double origin_x = x->current->settings.OriginX.real*M_TO_PS;
+      double origin_y = x->current->settings.OriginY.real*M_TO_PS;
+      for (i=0;i<8;i++)
+       {
+        xap=((i&1)!=0);
+        yap=((i&2)!=0);
+        zap=((i&4)!=0);
+        eps_plot_ThreeDimProject(xap,yap,zap,&x->current->settings,origin_x,origin_y,width,height,zdepth,data+3*i,data+3*i+1,data+3*i+2);
+       }
+      SortByAzimuthXCentre = origin_x;
+      SortByAzimuthYCentre = origin_y;
+      qsort((void *)(data  ),8,3*sizeof(double),SortByDepth);
+      if ((data[3*0+2]==data[3*1+2])&&(hypot(data[3*0+0]-origin_x,data[3*0+1]-origin_y)>hypot(data[3*1+0]-origin_x,data[3*1+1]-origin_y))) TriSwap(data+3*0,data+3*1);
+      if ((data[3*7+2]==data[3*6+2])&&(hypot(data[3*7+0]-origin_x,data[3*7+1]-origin_y)>hypot(data[3*6+0]-origin_x,data[3*6+1]-origin_y))) TriSwap(data+3*7,data+3*6);
+      qsort((void *)(data+3),6,3*sizeof(double),SortByAzimuthProximity);
+      if ((kp==SW_KEYPOS_TM)||(kp==SW_KEYPOS_BM))
+        xoff += data[3*1+0];
+      else
+        yoff += data[3*1+1];
+     }
+   }
+ 
+  xoff += x->current->settings.KeyXOff.real * M_TO_PS;
+  yoff += x->current->settings.KeyYOff.real * M_TO_PS;   
   LOOP_OVER_DATASETS;
   pd->TitleFinal_xpos += xoff;
   pd->TitleFinal_ypos += yoff;
@@ -313,7 +405,7 @@ void GraphLegend_Render(EPSComm *x, double width, double height, double zdepth)
     if (x->current->settings.TextColour > 0) { ww.colour = x->current->settings.TextColour; ww.USEcolour = 1; }
     else                                     { ww.Col1234Space = x->current->settings.TextCol1234Space; ww.colour1 = x->current->settings.TextColour1; ww.colour2 = x->current->settings.TextColour2; ww.colour3 = x->current->settings.TextColour3; ww.colour4 = x->current->settings.TextColour4; ww.USEcolour1234 = 1; }
     eps_core_SetColour(x, &ww, 1);
-    IF_NOT_INVISIBLE canvas_EPSRenderTextItem(x, pageno, (pd->TitleFinal_xpos+MARGIN_HSIZE_LEFT)/M_TO_PS, (pd->TitleFinal_ypos - pd->TitleFinal_height/2)/ M_TO_PS, SW_HALIGN_LEFT, SW_VALIGN_CENT, x->CurrentColour, fs, 0.0, NULL, NULL);
+    IF_NOT_INVISIBLE canvas_EPSRenderTextItem(x, NULL, pageno, (pd->TitleFinal_xpos+MARGIN_HSIZE_LEFT)/M_TO_PS, (pd->TitleFinal_ypos - pd->TitleFinal_height/2)/ M_TO_PS, SW_HALIGN_LEFT, SW_VALIGN_CENT, x->CurrentColour, fs, 0.0, NULL, NULL);
    NDataSet++;
   END_LOOP_OVER_DATASETS
   return;
