@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: eps_plot_ticking.c 814 2010-05-28 15:14:41Z dcf21 $
+// $Id: eps_plot_ticking.c 914 2010-07-29 16:11:37Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -72,18 +72,18 @@ void eps_plot_ticking(settings_axis *axis, int AxisUnitStyle, settings_axis *lin
     else                        axis->MaxFinal = (axis->LogFinal == SW_BOOL_TRUE) ? 10.0 : 10.0;
 
     // Check that log axes do not venture too close to zero
-    if ((axis->LogFinal == SW_BOOL_TRUE) && (axis->MaxFinal <= 1e-200)) { axis->MaxFinal = logmin; sprintf(temp_err_string, "Range for logarithmic axis %c%d set below zero; defaulting to 1e-10.", "xyz"[axis->xyz], axis->axis_n); ppl_warning(ERR_NUMERIC, temp_err_string); }
+    if ((axis->LogFinal == SW_BOOL_TRUE) && (axis->MaxFinal <= 1e-200)) { axis->MaxFinal = logmin; sprintf(temp_err_string, "Range for logarithmic axis %c%d set below zero; defaulting to 1e-10.", "xyzc"[axis->xyz], axis->axis_n); ppl_warning(ERR_NUMERIC, temp_err_string); }
     if (!MinSet) axis->MinFinal = (axis->LogFinal == SW_BOOL_TRUE) ? (axis->MaxFinal / 100) : (axis->MaxFinal - 20);
-    if ((axis->LogFinal == SW_BOOL_TRUE) && (axis->MinFinal <= 1e-200)) { axis->MinFinal = logmin; sprintf(temp_err_string, "Range for logarithmic axis %c%d set below zero; defaulting to 1e-10.", "xyz"[axis->xyz], axis->axis_n); ppl_warning(ERR_NUMERIC, temp_err_string); }
+    if ((axis->LogFinal == SW_BOOL_TRUE) && (axis->MinFinal <= 1e-200)) { axis->MinFinal = logmin; sprintf(temp_err_string, "Range for logarithmic axis %c%d set below zero; defaulting to 1e-10.", "xyzc"[axis->xyz], axis->axis_n); ppl_warning(ERR_NUMERIC, temp_err_string); }
 
     // If there's no spread of data on the axis, make a spread up
     if ( (fabs(axis->MinFinal-axis->MaxFinal) <= fabs(1e-14*axis->MinFinal)) || (fabs(axis->MinFinal-axis->MaxFinal) <= fabs(1e-14*axis->MaxFinal)) )
      {
-      if (axis->HardMinSet && axis->HardMaxSet) { sprintf(temp_err_string, "Specified minimum and maximum range limits for axis %c%d are equal; reverting to alternative limits.", "xyz"[axis->xyz], axis->axis_n); ppl_warning(ERR_NUMERIC, temp_err_string); }
+      if (axis->HardMinSet && axis->HardMaxSet) { sprintf(temp_err_string, "Specified minimum and maximum range limits for axis %c%d are equal; reverting to alternative limits.", "xyzc"[axis->xyz], axis->axis_n); ppl_warning(ERR_NUMERIC, temp_err_string); }
       if (axis->LogFinal != SW_BOOL_TRUE)
        {
-        axis->MinFinal -= max(1.0,1e-3*fabs(axis->MinFinal));
-        axis->MaxFinal += max(1.0,1e-3*fabs(axis->MinFinal));
+        double step = max(1.0,1e-3*fabs(axis->MinFinal));
+        axis->MinFinal -= step; axis->MaxFinal += step;
        }
       else
        {
@@ -104,6 +104,7 @@ void eps_plot_ticking(settings_axis *axis, int AxisUnitStyle, settings_axis *lin
   CentralValue.FlagComplex = 0;
   CentralValue.imag = 0.0;
   CentralValue.real = (axis->format==NULL) ? eps_plot_axis_InvGetPosition(0.5, axis) : 1.0;
+  if (CentralValue.real==0.0) CentralValue.real = eps_plot_axis_InvGetPosition(0.25, axis);
   UnitString = ppl_units_GetUnitStr(&CentralValue,&UnitMultiplier,NULL,0,0,SW_DISPLAY_L);
   UnitMultiplier /= CentralValue.real;
   if (!gsl_finite(UnitMultiplier)) UnitMultiplier=1.0;
@@ -133,7 +134,7 @@ void eps_plot_ticking(settings_axis *axis, int AxisUnitStyle, settings_axis *lin
     // Print out debugging report
     if (DEBUG)
      {
-      sprintf(temp_err_string,"Determined range for axis %c%d of plot %d. Usage was [", "xyz"[axis->xyz], axis->axis_n, axis->canvas_id);
+      sprintf(temp_err_string,"Determined range for axis %c%d of plot %d. Usage was [", "xyzc"[axis->xyz], axis->axis_n, axis->canvas_id);
       i = strlen(temp_err_string);
       if (axis->MinUsedSet) { sprintf(temp_err_string+i, "%f", axis->MinUsed); i+=strlen(temp_err_string+i); }
       else                  temp_err_string[i++] = '*';
@@ -159,12 +160,11 @@ void eps_plot_ticking(settings_axis *axis, int AxisUnitStyle, settings_axis *lin
   if (!axis->TickListFinalised)
    {
     int OutContext;
-    double tick_sep_major , tick_sep_minor;
 
     // If ticks have been manually specified, check that units are right
     if ((!ppl_units_DimEqual(&axis->unit,&axis->DataUnit)) && ((axis->TickList!=NULL)||(((axis->log==SW_BOOL_TRUE)?(axis->TickMinSet):(axis->TickStepSet))!=0)||(axis->MTickList!=NULL)||(((axis->log==SW_BOOL_TRUE)?(axis->MTickMinSet):(axis->MTickStepSet))!=0)))
      {
-      sprintf(temp_err_string, "Cannot put any ticks on axis %c%d because their positions are specified in units of <%s> whilst the axis has units of <%s>.", "xyz"[axis->xyz], axis->axis_n, ppl_units_GetUnitStr(&axis->unit,NULL,NULL,0,1,0), ppl_units_GetUnitStr(&axis->DataUnit,NULL,NULL,1,1,0));
+      sprintf(temp_err_string, "Cannot put any ticks on axis %c%d because their positions are specified in units of <%s> whilst the axis has units of <%s>.", "xyzc"[axis->xyz], axis->axis_n, ppl_units_GetUnitStr(&axis->unit,NULL,NULL,0,1,0), ppl_units_GetUnitStr(&axis->DataUnit,NULL,NULL,1,1,0));
       ppl_error(ERR_GENERAL,-1,-1,temp_err_string);
       axis-> TickListPositions = NULL; axis-> TickListStrings = NULL;
       axis->MTickListPositions = NULL; axis->MTickListStrings = NULL;
@@ -173,18 +173,6 @@ void eps_plot_ticking(settings_axis *axis, int AxisUnitStyle, settings_axis *lin
 
     OutContext = lt_GetMemContext();
 
-    // Work out optimal tick separation
-    if (axis->xyz!=1)
-     {
-      tick_sep_major = 0.025;
-      tick_sep_minor = 0.004;
-     }
-    else
-     {
-      tick_sep_major = 0.015;
-      tick_sep_minor = 0.004;
-     }
-
     // Finalise the label to be placed on the axis, quoting a physical unit as necessary
     if ((axis->DataUnit.dimensionless) || (axis->format != NULL))
      { axis->FinalAxisLabel = axis->label; } // No units to append
@@ -235,18 +223,16 @@ void eps_plot_ticking(settings_axis *axis, int AxisUnitStyle, settings_axis *lin
       else if (TickStepSet)
        {
         double TMin, TStep, TMax, tmp;
+        unsigned char inverted=0;
         *TickListPositions = (double  *)lt_malloc(102 * (axis->AxisValueTurnings+1) * sizeof(double));
         *TickListStrings   = (char   **)lt_malloc(102 * (axis->AxisValueTurnings+1) * sizeof(char *));
         if ((*TickListPositions==NULL) || (*TickListStrings==NULL)) { ppl_error(ERR_MEMORY, -1, -1, "Out of memory"); *TickListPositions = NULL; *TickListStrings = NULL; return; }
         TStep= TickStep;
-        if (TStep<0) TStep=-TStep;
-        if (axis->LogFinal == SW_BOOL_TRUE) { if (TStep<1) TStep=1/TStep; }
+        if (TStep<0) { TStep=-TStep; inverted=1; }
+        if (axis->LogFinal == SW_BOOL_TRUE) { if (TStep<1) { TStep=1/TStep; inverted=1; } else inverted=0; }
 
         for (xrn=j=0; xrn<=axis->AxisValueTurnings; xrn++)
          {
-          TMin = TickMinSet ? TickMin : axis->MinFinal;
-          TMax = TickMaxSet ? TickMax : axis->MaxFinal;
-
           if (axis->AxisLinearInterpolation!=NULL)
            {
             double RegionMin, RegionMax, first;
@@ -255,31 +241,48 @@ void eps_plot_ticking(settings_axis *axis, int AxisUnitStyle, settings_axis *lin
             RegionMax = axis->AxisLinearInterpolation[axis->AxisTurnings[xrn+1]];
             if (RegionMax<RegionMin) { tmp=RegionMin; RegionMin=RegionMax; RegionMax=tmp; }
 
-            if (!TickMaxSet) TMax = RegionMax;
-            if (!TickMinSet)
+            if (!inverted) { TMin = TickMinSet ? TickMin : first;  TMax = TickMaxSet ? TickMax : RegionMax; }
+            else           { TMin = TickMaxSet ? TickMax : first;  TMax = TickMinSet ? TickMin : RegionMax; }
+
+            if ((TMin < RegionMin) || ((!inverted)&&(!TickMinSet)) || ((inverted)&&(!TickMaxSet)))
              {
-              if (axis->LogFinal == SW_BOOL_TRUE) TMin = first - floor((TMin-RegionMin)/TStep) * TStep;
-              else                                TMin = exp(log(first) - floor((log(TMin)-log(RegionMin))/log(TStep)) * log(TStep));
+              if (axis->LogFinal == SW_BOOL_TRUE) TMin *= exp(ceil ((log(RegionMin / TMin))/log(TStep)) * log(TStep));
+              else                                TMin +=     ceil ((    RegionMin - TMin )/    TStep ) *     TStep  ;
+             }
+            if (TMax > RegionMax)
+             {
+              if (axis->LogFinal == SW_BOOL_TRUE) TMax /= exp(floor((log(TMax / RegionMax))/log(TStep)) * log(TStep));
+              else                                TMax -=     floor((    TMax - RegionMax )/    TStep ) *     TStep  ;
              }
-            if (TMax < TMin) { tmp=TMax; TMax=TMin; TMin=tmp; }
            }
           else
            {
-            if (TMax < TMin) { tmp=TMax; TMax=TMin; TMin=tmp; }
-            if (TMin < axis->MinFinal)
+            if (!inverted) { TMin = TickMinSet ? TickMin : min(axis->MinFinal , axis->MaxFinal);  TMax = TickMaxSet ? TickMax : max(axis->MinFinal , axis->MaxFinal); }
+            else           { TMin = TickMaxSet ? TickMax : min(axis->MinFinal , axis->MaxFinal);  TMax = TickMinSet ? TickMin : max(axis->MinFinal , axis->MaxFinal); }
+
+            if (TMin < min(axis->MinFinal , axis->MaxFinal))
              {
-              if (axis->LogFinal == SW_BOOL_TRUE) TMin += exp(ceil ((log(axis->MinFinal)-log(TMin))/log(TStep)) * log(TStep));
-              else                                TMin +=     ceil ((    axis->MinFinal -    TMin )/    TStep ) *     TStep  ;
+              if (axis->LogFinal == SW_BOOL_TRUE) TMin *= exp(ceil ((log(axis->MinFinal / TMin))/log(TStep)) * log(TStep));
+              else                                TMin +=     ceil ((    axis->MinFinal - TMin )/    TStep ) *     TStep  ;
              }
-            if (TMax > axis->MaxFinal)
+            if (TMax > max(axis->MinFinal , axis->MaxFinal))
              {
-              if (axis->LogFinal == SW_BOOL_TRUE) TMax -= exp(floor((log(TMax)-log(axis->MaxFinal))/log(TStep)) * log(TStep));
-              else                                TMax -=     floor((    TMax -    axis->MaxFinal )/    TStep ) *     TStep  ;
+              if (axis->LogFinal == SW_BOOL_TRUE) TMax /= exp(floor((log(TMax / axis->MaxFinal))/log(TStep)) * log(TStep));
+              else                                TMax -=     floor((    TMax - axis->MaxFinal )/    TStep ) *     TStep  ;
              }
            }
-          tmp = TMin;
-          for (i=0; (i<100)&&(tmp<=TMax); i++)
+          for (i=0; i<100; i++)
            {
+            if (!inverted)
+             {
+              if (axis->LogFinal == SW_BOOL_TRUE) tmp = TMin * pow(TStep, i);
+              else                                tmp = TMin + i*TStep;
+             } else {
+              if (axis->LogFinal == SW_BOOL_TRUE) tmp = TMax * pow(TStep,-i);
+              else                                tmp = TMax - i*TStep;
+             }
+            if (((!inverted)&&(tmp>TMax)) || ((inverted)&&(tmp<TMin))) break;
+            if ((axis->LogFinal != SW_BOOL_TRUE) && (fabs(tmp)<fabs(TStep*1e-14))) tmp=0; // Enforce that ticks close to zero are at zero
             (*TickListPositions)[j] = eps_plot_axis_GetPosition(tmp, axis, xrn, 0);
             if ( (!gsl_finite((*TickListPositions)[j])) || ((*TickListPositions)[j]<0.0) || ((*TickListPositions)[j]>1.0) ) continue; // Filter out ticks which are off the end of the axis
             if      (MajMin==0)            (*TickListStrings)[j] = "";
@@ -287,7 +290,6 @@ void eps_plot_ticking(settings_axis *axis, int AxisUnitStyle, settings_axis *lin
             else                           TickLabelFromFormat(&(*TickListStrings)[j], axis->format, tmp, &axis->DataUnit, axis->xyz, OutContext);
             if ((*TickListStrings)[j]==NULL) { ppl_error(ERR_MEMORY, -1, -1, "Out of memory"); *TickListPositions = NULL; *TickListStrings = NULL; return; }
             j++;
-            if (axis->LogFinal == SW_BOOL_TRUE) tmp*=TStep; else tmp+=TStep;
            }
          }
         (*TickListStrings)[j] = NULL; // null terminate list
@@ -302,11 +304,11 @@ void eps_plot_ticking(settings_axis *axis, int AxisUnitStyle, settings_axis *lin
     if (AutoTicks[1])
      {
       if ((axis->format != NULL) || (axis->AxisLinearInterpolation != NULL))
-        eps_plot_ticking_auto (axis, UnitMultiplier, AutoTicks, tick_sep_major, tick_sep_minor, linkedto);
+        eps_plot_ticking_auto (axis, UnitMultiplier, AutoTicks, linkedto);
       else
        {
-        if (linkedto!=NULL) eps_plot_ticking_auto3(axis, UnitMultiplier, AutoTicks, tick_sep_major, tick_sep_minor, linkedto);
-        else                eps_plot_ticking_auto2(axis, UnitMultiplier, AutoTicks, tick_sep_major, tick_sep_minor, linkedto);
+        if (linkedto!=NULL) eps_plot_ticking_auto3(axis, UnitMultiplier, AutoTicks, linkedto);
+        else                eps_plot_ticking_auto2(axis, UnitMultiplier, AutoTicks, linkedto);
        }
      }
 
@@ -325,7 +327,7 @@ void TickLabelAutoGen(char **output, double x, double log_base, int OutContext)
   ApproxMargin = pow(10,-SF+1);
   if (ApproxMargin < 1e-15) ApproxMargin = 1e-15;
 
-  if ((fabs(x)<DBL_MIN*100) || ((fabs(x)>=1e-3) && (fabs(x)<1e5))) { sprintf(temp_err_string,"%s",NumericDisplay(x,0,SF,1)); }
+  if ((fabs(x)<DBL_MIN*100) || ((fabs(x)>1e-3) && (fabs(x)<1e5))) { sprintf(temp_err_string,"%s",NumericDisplay(x,0,SF,1)); }
   else
    {
     double e,m;
@@ -350,7 +352,8 @@ void TickLabelFromFormat(char **output, char *FormatStr, double x, value *xunit,
   value DummyTemp, *VarVal;
   if      (xyz==0) VarName = "x";
   else if (xyz==1) VarName = "y";
-  else             VarName = "z";
+  else if (xyz==2) VarName = "z";
+  else             VarName = "c";
 
   // Look up variable in user space and get pointer to its value
   DictLookup(_ppl_UserSpace_Vars, VarName, NULL, (void **)&VarVal);
