@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: eps_plot_ticking_auto.c 813 2010-05-27 17:04:57Z dcf21 $
+// $Id: eps_plot_ticking_auto.c 905 2010-07-25 19:31:16Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -181,7 +181,7 @@ static void TickScheme_sprintf(char *out, const PotentialTick *PotTickList, cons
  }
 
 // Add a tick scheme to a list of accepted ticks
-static void AddTickScheme(const PotentialTick *PotTickList, const int NPotTicks, const ArgLeagueTableEntry *TickOrder, const int NIntervals, const int ArgNo, const int DivOfThrow, const int OoM, const int DivOfOoM, const unsigned char *TicksAcceptedIn, unsigned char *TicksAcceptedOut, const unsigned char MAJORminor, const double AxisLength, const double TickSepMin, unsigned char *FLAGacceptable, int *NTicksOutMajor, int *NTicksOutMinor)
+static void AddTickScheme(const PotentialTick *PotTickList, const int NPotTicks, const ArgLeagueTableEntry *TickOrder, const int NIntervals, const int ArgNo, const int DivOfThrow, const int OoM, const int DivOfOoM, const unsigned char *TicksAcceptedIn, unsigned char *TicksAcceptedOut, const unsigned char MAJORminor, const double AxisLength, unsigned char *FLAGacceptable, int *NTicksOutMajor, int *NTicksOutMinor)
  {
   int i, j, id, IntNum, abort, imin=-1, imax=-1;
   *NTicksOutMajor = 0;
@@ -238,7 +238,7 @@ static void AddTickScheme(const PotentialTick *PotTickList, const int NPotTicks,
        if (TicksAcceptedOut[j]==1)
         {
          double gap = fabs(PotTickList[id].IntervalNum - PotTickList[TickOrder[j].id].IntervalNum) * AxisLength / (NIntervals-1);
-         if (gap < TickSepMin) *FLAGacceptable = 0; // Gap between this tick and that to our left is too small
+         if (gap < 1) *FLAGacceptable = 0; // Gap between this tick and that to our left is too small
          break;
         }
      }
@@ -252,7 +252,7 @@ static void AddTickScheme(const PotentialTick *PotTickList, const int NPotTicks,
       if ((TicksAcceptedOut[i] != 0) && (id>=imin) && (id<=imax)) Nnewticks++;
      }
     if   (Nnewticks <= 1) *FLAGacceptable = 1;
-    else                  *FLAGacceptable = (unsigned char)((Nnewticks>0) && (AxisLength*(imax-imin)/(NIntervals-1)/(Nnewticks-1) > TickSepMin));
+    else                  *FLAGacceptable = (unsigned char)((Nnewticks>0) && (AxisLength*(imax-imin)/(NIntervals-1)/(Nnewticks-1) > 1));
    }
   return;
  }
@@ -378,7 +378,7 @@ FAIL:
  }
 
 // Main entry point for automatic ticking of axes
-void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned char *AutoTicks, double tick_sep_major, double tick_sep_minor, settings_axis *linkedto)
+void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned char *AutoTicks, settings_axis *linkedto)
  {
   int    i, j, k, l, start, NArgs, OutContext, ContextRough=-1, CommaPositions[MAX_ARGS+2], NFactorsLogBase, LogBase;
   int    FactorsLogBase[MAX_FACTORS];
@@ -392,7 +392,7 @@ void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned
   unsigned char *TicksAccepted, *TicksAcceptedNew, *TicksAcceptedNewTF, *TicksAcceptedNew2B, *TicksAcceptedRough, *TicksAcceptedRough2;
 
   if (DEBUG) ppl_log("Using eps_plot_ticking_auto()");
-  N_STEPS = (2 + axis->PhysicalLength/tick_sep_major) * STEP_DUPLICITY; // Number of intervals into which we divide axis
+  N_STEPS = (2 + axis->PhysicalLengthMajor) * STEP_DUPLICITY; // Number of intervals into which we divide axis
 
   // Make temporary rough workspace
   OutContext   = lt_GetMemContext();
@@ -401,10 +401,10 @@ void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned
   if (DummyStr==NULL) goto FAIL;
 
   // Work through format string identifying substitution expressions
-  VarName[0] = "xyz"[axis->xyz];
+  VarName[0] = "xyzc"[axis->xyz];
   if (axis->format == NULL) { sprintf(FormatTemp, "\"%%s\"%%(%s)", VarName); format=FormatTemp; }
   else                      { format=axis->format; }
-  if ((format==FormatTemp) && (axis->AxisLinearInterpolation==NULL) && (axis->LogFinal==SW_BOOL_TRUE) && (log(axis->MaxFinal / axis->MinFinal) / log(axis->LogBase) > axis->PhysicalLength / tick_sep_major)) { sprintf(FormatTemp, "\"%%s\"%%(logn(%s,%f))", VarName, axis->LogBase); }
+  if ((format==FormatTemp) && (axis->AxisLinearInterpolation==NULL) && (axis->LogFinal==SW_BOOL_TRUE) && (log(axis->MaxFinal / axis->MinFinal) / log(axis->LogBase) > axis->PhysicalLengthMajor)) { sprintf(FormatTemp, "\"%%s\"%%(logn(%s,%f))", VarName, axis->LogBase); }
   for (i=0; ((format[i]!='\0')&&(format[i]!='\'')&&(format[i]!='\"')); i++);
   QuoteType = format[i];
   if (QuoteType=='\0') goto FAIL;
@@ -508,7 +508,7 @@ void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned
 
   // Work out factors of log base of axis. In fact, work out factors of log base ** 2, so that ten divides by four.
   LogBase = (axis->LogFinal == SW_BOOL_TRUE) ? axis->LogBase : 10;
-  factorise(pow(LogBase,FACTOR_MULTIPLY), FactorsLogBase, MAX_FACTORS, axis->PhysicalLength/tick_sep_minor*1.2, &NFactorsLogBase);
+  factorise(pow(LogBase,FACTOR_MULTIPLY), FactorsLogBase, MAX_FACTORS, axis->PhysicalLengthMinor*1.2, &NFactorsLogBase);
 
   // Work out throw of each argument (i.e. the spread of values that it takes)
   for (i=0; i<NArgs; i++)
@@ -520,7 +520,7 @@ void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned
     if ((throw>1e6) || (args[i].MinValue < INT_MIN+10) || (args[i].MaxValue > INT_MAX-10)) args[i].Throw = 0.0;
     else                                                                                   args[i].Throw = throw;
     args[i].MinValue = floor(args[i].MinValue);
-    factorise(args[i].Throw*pow(LogBase,FACTOR_MULTIPLY-1), args[i].FactorsThrow, MAX_FACTORS, axis->PhysicalLength/tick_sep_minor*1.2, &args[i].NFactorsThrow);
+    factorise(args[i].Throw*pow(LogBase,FACTOR_MULTIPLY-1), args[i].FactorsThrow, MAX_FACTORS, axis->PhysicalLengthMinor*1.2, &args[i].NFactorsThrow);
    }
 
   // Compile scores of how fast each continuous argument moves
@@ -579,7 +579,7 @@ void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned
          const double OoM_m  = log(fabs(args[i].NumericValues[j  ])) / log(LogBase);
          const int    Nsteps = fabs(log(1e-15) / log(LogBase));
          double OoM, nd;
-         int    n,m,ZeroInInterval=0;
+         int    n=0,m,ZeroInInterval=0;
          if      (!gsl_finite(OoM_n)) OoM = floor(OoM_m);
          else if (!gsl_finite(OoM_m)) OoM = floor(OoM_n);
          else                         OoM = floor(max(OoM_n , OoM_m));
@@ -604,6 +604,7 @@ void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned
             )
          {
           ADDTICK(i,NOT_THROW,INT_MAX,-10,0.0,j); // Zero should be marked in this interval
+          n=0;
           ZeroInInterval = 1;
          }
          else if ((args[i].NumericValues[j-1]!=0)||(args[i].NumericValues[j]!=0))
@@ -657,7 +658,6 @@ void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned
          }
 
          // Ticks which mark the passing of fractions of the Nth significant digit
-         if (ZeroInInterval) n=0;
          if (n<Nsteps)
            for (m=n; m>=n-((ZeroInInterval||(j==1))?Nsteps:3); m--)
             {
@@ -735,7 +735,7 @@ void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned
 
        // Try marking discrete changes to arguments with major ticks
        AddTickScheme(PotTickList, NPotTicks, TickOrder, N_STEPS, i, 0, 0, 0, TicksAccepted, TicksAcceptedNew,
-                     1, axis->PhysicalLength, tick_sep_major, &acceptable, &NTicksMajor, &NTicksMinor);
+                     1, axis->PhysicalLengthMajor, &acceptable, &NTicksMajor, &NTicksMinor);
        if (acceptable)
         {
          if (DEBUG) { sprintf(temp_err_string, "Accepted major ticks to mark changes of argument %d (now %d ticks).", i, NTicksMajor); ppl_log(temp_err_string); }
@@ -745,7 +745,7 @@ void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned
 
        // Try marking discrete changes to arguments with minor ticks
        AddTickScheme(PotTickList, NPotTicks, TickOrder, N_STEPS, i, 0, 0, 0, TicksAccepted, TicksAcceptedNew,
-                     0, axis->PhysicalLength, tick_sep_minor, &acceptable, &NTicksMajor, &NTicksMinor);
+                     0, axis->PhysicalLengthMinor, &acceptable, &NTicksMajor, &NTicksMinor);
        if (acceptable)
         {
          if (DEBUG) { sprintf(temp_err_string, "Accepted minor ticks to mark changes of argument %d (now %d ticks).", i, NTicksMajor); ppl_log(temp_err_string); }
@@ -760,7 +760,7 @@ void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned
        int Nticks_new, Nticks_new_minor, Nticks_new_accepted, Nticks_new_accepted_minor;
        int Nticks_new_prev, Nticks_new_prev_minor, Nticks_2B, Nticks_2B_minor;
        int Nticks_2B_best=-1, Nticks_2B_minor_best=-1;
-       int l, l2, l3, l4, l_final;
+       int l, l2, l3, l4, l_final=0;
        int NMajorTicksIn, NMinorTicksIn;
        unsigned char MAJORminor;
 
@@ -769,7 +769,7 @@ void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned
         {
          acceptable = 1;
          AddTickScheme(PotTickList, NPotTicks, TickOrder, N_STEPS, i, k, 0, 0, TicksAccepted, TicksAcceptedNewTF,
-                       1, axis->PhysicalLength, tick_sep_major, &acceptable, &ThrowFactors_Nticks, &ThrowFactors_Nticks_minor);
+                       1, axis->PhysicalLengthMajor, &acceptable, &ThrowFactors_Nticks, &ThrowFactors_Nticks_minor);
          if (DEBUG) { sprintf(temp_err_string, "Dividing throw of %3d into intervals of %.1f produces %d major ticks [%s].",args[i].Throw,args[i].FactorsThrow[k]/pow(LogBase,FACTOR_MULTIPLY-1),ThrowFactors_Nticks,acceptable?"pass":"fail"); ppl_log(temp_err_string); }
          if (acceptable) { ThrowFactors_FactNum =  k; break; }
          else            { ThrowFactors_FactNum = -1; ThrowFactors_Nticks  = -1; }
@@ -782,7 +782,7 @@ void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned
           {
            acceptable=1;
            AddTickScheme(PotTickList, NPotTicks, TickOrder, N_STEPS, i, k, 0, 0, TicksAcceptedNewTF, TicksAcceptedRough,
-                         0, axis->PhysicalLength, tick_sep_minor, &acceptable, &ThrowFactors_Nticks, &ThrowFactors_Nticks_minor);
+                         0, axis->PhysicalLengthMinor, &acceptable, &ThrowFactors_Nticks, &ThrowFactors_Nticks_minor);
          if (DEBUG) { sprintf(temp_err_string, "Dividing throw of %3d into intervals of %.1f produces %d minor ticks [%s].",args[i].Throw,args[i].FactorsThrow[k]/pow(LogBase,FACTOR_MULTIPLY-1),ThrowFactors_Nticks_minor,acceptable?"pass":"fail"); ppl_log(temp_err_string); }
            if (acceptable) { memcpy(TicksAcceptedNewTF, TicksAcceptedRough, NPotTicks); break; }
           }
@@ -807,8 +807,8 @@ void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned
              if (Nticks_2B>=0) break;
              l_final = l;
              acceptable=1;
-             if (!l2)        AddTickScheme(PotTickList, NPotTicks, TickOrder, N_STEPS, i, NOT_THROW, k, 2*(l+1)-1, TicksAcceptedNew, TicksAcceptedRough, 1, axis->PhysicalLength, tick_sep_major, &acceptable, &Nticks_2B, &Nticks_2B_minor);
-             if (acceptable) AddTickScheme(PotTickList, NPotTicks, TickOrder, N_STEPS, i, NOT_THROW, k, 2*(l+1), l2?TicksAcceptedNew:TicksAcceptedRough, TicksAcceptedRough, 1, axis->PhysicalLength, tick_sep_major, &acceptable, &Nticks_2B, &Nticks_2B_minor);
+             if (!l2)        AddTickScheme(PotTickList, NPotTicks, TickOrder, N_STEPS, i, NOT_THROW, k, 2*(l+1)-1, TicksAcceptedNew, TicksAcceptedRough, 1, axis->PhysicalLengthMajor, &acceptable, &Nticks_2B, &Nticks_2B_minor);
+             if (acceptable) AddTickScheme(PotTickList, NPotTicks, TickOrder, N_STEPS, i, NOT_THROW, k, 2*(l+1), l2?TicksAcceptedNew:TicksAcceptedRough, TicksAcceptedRough, 1, axis->PhysicalLengthMajor, &acceptable, &Nticks_2B, &Nticks_2B_minor);
              if (DEBUG) { sprintf(temp_err_string, "Dividing OoM %g into intervals of %g produces %d major ticks [%s%s].",pow(LogBase,k),FactorsLogBase[l ]*pow(LogBase,k-FACTOR_MULTIPLY),Nticks_2B,acceptable?"pass":"fail",(Nticks_2B<=NMajorTicksIn)?"; no new ticks":""); ppl_log(temp_err_string); }
              if ((!acceptable) || (Nticks_2B<=NMajorTicksIn)) { Nticks_2B=-1; Nticks_2B_minor=-1; }
             }
@@ -823,8 +823,8 @@ void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned
                continue; // Minor ticks must mark common factors of log base and major tick interval
               }
              acceptable=1;
-             if (!l4)        AddTickScheme(PotTickList, NPotTicks, TickOrder, N_STEPS, i, NOT_THROW, k, 2*(l3+1)-1, TicksAcceptedRough, TicksAcceptedRough2, 0, axis->PhysicalLength, tick_sep_minor, &acceptable, &Nticks_2B, &Nticks_2B_minor);
-             if (acceptable) AddTickScheme(PotTickList, NPotTicks, TickOrder, N_STEPS, i, NOT_THROW, k, 2*(l3+1), l4?TicksAcceptedRough:TicksAcceptedRough2, TicksAcceptedRough2, 0, axis->PhysicalLength, tick_sep_minor, &acceptable, &Nticks_2B, &Nticks_2B_minor);
+             if (!l4)        AddTickScheme(PotTickList, NPotTicks, TickOrder, N_STEPS, i, NOT_THROW, k, 2*(l3+1)-1, TicksAcceptedRough, TicksAcceptedRough2, 0, axis->PhysicalLengthMinor, &acceptable, &Nticks_2B, &Nticks_2B_minor);
+             if (acceptable) AddTickScheme(PotTickList, NPotTicks, TickOrder, N_STEPS, i, NOT_THROW, k, 2*(l3+1), l4?TicksAcceptedRough:TicksAcceptedRough2, TicksAcceptedRough2, 0, axis->PhysicalLengthMinor, &acceptable, &Nticks_2B, &Nticks_2B_minor);
              if (DEBUG) { sprintf(temp_err_string, "Dividing OoM %g into intervals of %g produces %d minor ticks [%s%s].",pow(LogBase,k),FactorsLogBase[l3]*pow(LogBase,k-FACTOR_MULTIPLY),Nticks_2B_minor,acceptable?"pass":"fail",(Nticks_2B_minor<=NMinorTicksIn)?"; no new ticks":""); ppl_log(temp_err_string); }
              if (!acceptable) Nticks_2B_minor = NMinorTicksIn;
             }
@@ -848,7 +848,7 @@ void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned
            if (k==args[i].OoM_max+1)
             {
              AddTickScheme(PotTickList, NPotTicks, TickOrder, N_STEPS, i, NOT_THROW, INT_MAX, -10, TicksAcceptedNew, TicksAcceptedRough,
-                           1, axis->PhysicalLength, tick_sep_major, &acceptable, &Nticks_new, &Nticks_new_minor); // Must have zero, if present, with highest OoM
+                           1, axis->PhysicalLengthMajor, &acceptable, &Nticks_new, &Nticks_new_minor); // Must have zero, if present, with highest OoM
              if (acceptable) memcpy(TicksAcceptedNew, TicksAcceptedRough, NPotTicks);
             }
            memcpy(TicksAcceptedRough, TicksAcceptedNew, NPotTicks);
@@ -858,7 +858,7 @@ void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned
            for (l=-3; l<=0; l++)
             {
              AddTickScheme(PotTickList, NPotTicks, TickOrder, N_STEPS, i, NOT_THROW, k, l, TicksAcceptedNew, TicksAcceptedRough,
-                           MAJORminor, axis->PhysicalLength, MAJORminor ? tick_sep_major : tick_sep_minor, &acceptable, &Nticks_new, &Nticks_new_minor);
+                           MAJORminor, MAJORminor ? axis->PhysicalLengthMajor : axis->PhysicalLengthMinor, &acceptable, &Nticks_new, &Nticks_new_minor);
              if (DEBUG) { sprintf(temp_err_string, "OoM %g Priority %2d produces %d %s ticks [%s].",pow(LogBase,k),l,Nticks_new,MAJORminor?"major":"minor",acceptable?"pass":"fail"); ppl_log(temp_err_string); }
              if (acceptable) { Nticks_new_accepted=Nticks_new; Nticks_new_accepted_minor=Nticks_new_minor; memcpy(TicksAcceptedNew, TicksAcceptedRough, NPotTicks); }
              else            { if (MAJORminor) { MAJORminor=0; l--; continue; } else { break; } }
@@ -934,8 +934,8 @@ FAIL:
   // When clever logic fails, revert to PyXPlot 0.7 ticking algorithm
   if (ContextRough>0) lt_AscendOutOfContext(ContextRough);
   ContextRough = -1;
-  if (linkedto!=NULL) eps_plot_ticking_auto3(axis, UnitMultiplier, AutoTicks, tick_sep_major, tick_sep_minor, linkedto);
-  else                eps_plot_ticking_auto2(axis, UnitMultiplier, AutoTicks, tick_sep_major, tick_sep_minor, linkedto);
+  if (linkedto!=NULL) eps_plot_ticking_auto3(axis, UnitMultiplier, AutoTicks, linkedto);
+  else                eps_plot_ticking_auto2(axis, UnitMultiplier, AutoTicks, linkedto);
 
 CLEANUP:
   // Restore original value of x (or y/z)
