@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: eps_plot_ticking_auto.h 813 2010-05-27 17:04:57Z dcf21 $
+// $Id: eps_plot_ticking_auto.h 826 2010-06-08 17:05:09Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -24,7 +24,7 @@
 
 #include "ppl_settings.h"
 
-void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned char *AutoTicks, double tick_sep_major, double tick_sep_minor, settings_axis *linkedto);
+void eps_plot_ticking_auto(settings_axis *axis, double UnitMultiplier, unsigned char *AutoTicks, settings_axis *linkedto);
 
 #endif
 
