@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: dcfmath.c 809 2010-05-21 15:28:01Z dcf21 $
+// $Id: dcfmath.c 935 2010-08-26 15:14:55Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -766,6 +766,14 @@ void dcfmath_expint(value *in1, value *in2, value *output, int *status, char *er
   CHECK_OUTPUT_OKAY;
  }
 
+void dcfmath_finite(value *in, value *output, int *status, char *errtext)
+ {
+  WRAPPER_INIT;
+  if ((settings_term_current.ComplexNumbers == SW_ONOFF_OFF) && (in->FlagComplex)) return;
+  if ((!gsl_finite(in->real)) || (!gsl_finite(in->imag))) return;
+  output->real = 1;
+ }
+
 void dcfmath_floor(value *in, value *output, int *status, char *errtext)
  {
   char *FunctionDescription = "floor(x)";
@@ -1154,6 +1162,34 @@ void dcfmath_pow (value *in1, value *in2, value *output, int *status, char *errt
   ppl_units_pow(in1, in2, output, status, errtext);
  }
 
+void dcfmath_prime(value *in, value *output, int *status, char *errtext)
+ {
+  char *FunctionDescription = "prime(x)";
+  CHECK_1NOTNAN;
+  CHECK_1INPUT_DIMLESS;
+  CHECK_NEEDINT(in, "x", "function's argument must be an integer in the range");
+  IF_1COMPLEX { QUERY_MUST_BE_REAL }
+  ELSE_REAL
+   {
+    long x = floor(in->real), m, n;
+    if (x<53) // Hardcode primes less than 53
+     {
+      if ((x==2)||(x==3)||(x==5)||(x==7)||(x==11)||(x==13)||(x==17)||(x==19)||(x==23)||(x==29)||(x==31)||(x==37)||(x==41)||(x==43)||(x==47)) { output->real = 1; return; }
+      else return;
+     }
+    if (((x%2)==0)||((x%3)==0)||((x%5)==0)||((x%7)==0)) return;
+    m = sqrt(x);
+    for (n=11; n<=m; n+=6)
+     {
+      if ((x% n   )==0) return;
+      if ((x%(n+2))==0) return;
+     }
+    output->real = 1;
+   }
+  ENDIF
+  CHECK_OUTPUT_OKAY;
+ }
+
 void dcfmath_radians(value *in, value *output, int *status, char *errtext)
  {
   char *FunctionDescription = "radians(x)";
@@ -1479,14 +1515,21 @@ void dcfmath_zernike(value *in1, value *in2, value *in3, value *in4, value *outp
       if (settings_term_current.ExplicitErrors == SW_ONOFF_ON) { *status=1; sprintf(errtext, "The function %s is only defined for -n<=m<=n.", FunctionDescription); return; }
       else { NULL_OUTPUT; }
      }
-    if ((n%2)!=(m%2)) return; // Defined to be zero
-    for (i=0; i<(1+(n-m)/2); i++)
+    if ((r<0)||(r>1))
      {
-      output->real += sgn * gsl_sf_fact(n-i) / ( gsl_sf_fact(i) * gsl_sf_fact((n+m)/2-i) * gsl_sf_fact((n-m)/2-i) ) * pow(r , n-2*i);
-      sgn*=-1;
+      output->real = GSL_NAN; // Defined only within the unit disk
+     }
+    else
+     {
+      if ((n%2)!=(m%2)) return; // Defined to be zero
+      for (i=0; i<(1+(n-m)/2); i++)
+       {
+        output->real += sgn * gsl_sf_fact(n-i) / ( gsl_sf_fact(i) * gsl_sf_fact((n+m)/2-i) * gsl_sf_fact((n-m)/2-i) ) * pow(r , n-2*i);
+        sgn*=-1;
+       }
+      if      (ms>0) output->real *= cos(m*in4->real);
+      else if (ms<0) output->real *= sin(m*in4->real);
      }
-    if (ms>0) output->real *= cos(m*in4->real);
-    else      output->real *= sin(m*in4->real);
    }
   ENDIF
   CHECK_OUTPUT_OKAY;
@@ -1514,12 +1557,19 @@ void dcfmath_zernikeR(value *in1, value *in2, value *in3, value *output, int *st
       if (settings_term_current.ExplicitErrors == SW_ONOFF_ON) { *status=1; sprintf(errtext, "The function %s is only defined for -n<=m<=n.", FunctionDescription); return; }
       else { NULL_OUTPUT; }
      }
-    if ((n%2)!=(m%2)) return; // Defined to be zero
-
-    for (i=0; i<(1+(n-m)/2); i++)
+    if ((r<0)||(r>1))
      {
-      output->real += sgn * gsl_sf_fact(n-i) / ( gsl_sf_fact(i) * gsl_sf_fact((n+m)/2-i) * gsl_sf_fact((n-m)/2-i) ) * pow(r , n-2*i);
-      sgn*=-1;
+      output->real = GSL_NAN; // Defined only within the unit disk
+     }
+    else
+     {
+      if ((n%2)!=(m%2)) return; // Defined to be zero
+
+      for (i=0; i<(1+(n-m)/2); i++)
+       {
+        output->real += sgn * gsl_sf_fact(n-i) / ( gsl_sf_fact(i) * gsl_sf_fact((n+m)/2-i) * gsl_sf_fact((n-m)/2-i) ) * pow(r , n-2*i);
+        sgn*=-1;
+       }
      }
    }
   ENDIF
