@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: asciidouble.c 762 2010-05-12 22:46:32Z dcf21 $
+// $Id: asciidouble.c 878 2010-07-09 15:39:16Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -149,7 +149,8 @@ char *NumericDisplay(double in, int N, int SigFig, int latex)
          {
           for (j=i,k=i+32;output[j]!='\0';j++) output[j+32]=output[j];
           output[j+32]='\0';
-          strcpy(output+i,"\\times10^{"); i+=strlen(output+i); // Replace e with times ten to the...
+          if ((i==1)&&(output[0]=='1')) { strcpy(output  ,       "10^{"); i =strlen(output  ); } // Don't output 1 times 10^3
+          else                          { strcpy(output+i,"\\times10^{"); i+=strlen(output+i); } // Replace e with times ten to the...
           k++; // FFW over the E
           if (output[k]=='+') k++; // We don't need to say +8... 8 will do
           for (l=0,j=k;output[j]!='\0';j++) { if ((output[j]>'0')&&(output[j]<='9')) l=1; if ((l==1)||(output[j]!='0')) output[i++]=output[j]; } // Turn -08 into -8
