@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: ppl_canvasdraw.c 781 2010-05-17 15:36:40Z dcf21 $
+// $Id: ppl_canvasdraw.c 953 2010-09-09 16:11:57Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -33,6 +33,8 @@
 #include <sys/select.h>
 #include <sys/wait.h>
 
+#include <gsl/gsl_math.h>
+
 #include "EPSMaker/dvi_read.h"
 #include "EPSMaker/eps_comm.h"
 #include "EPSMaker/eps_core.h"
@@ -121,7 +123,7 @@ static char *PS_PROLOG_TEXT = "\
 ";
 
 // Standard commandline flags to pass to ghostscript
-static char *GHOSTSCRIPT_STANDARD_FLAGS = "-dQUIET -dSAFER -dBATCH -dNOPAUSE -dEPSCrop";
+static char *GHOSTSCRIPT_STANDARD_FLAGS = "-dQUIET -dSAFER -P- -dBATCH -dNOPAUSE -dEPSCrop";
 
 // Table of the functions we call for each phase of the canvas drawing process for different object types
 static void(*ArrowHandlers[])(EPSComm *) = {NULL                       , NULL                    , NULL                     , NULL                , NULL                , eps_arrow_RenderEPS, NULL};
@@ -614,6 +616,12 @@ void canvas_EPSWrite(EPSComm *x)
   char LandscapifyText[FNAME_LENGTH], EnlargementText[FNAME_LENGTH], *PaperName, *PFAfilename;
   ListIterator *ListIter;
 
+  // Check that we have a bounding box
+  if (!x->bb_set)
+   {
+    x->bb_left = x->bb_right = x->bb_bottom = x->bb_top = 0.0;
+   }
+
   // Apply enlarge and landscape terminals as required
   LandscapifyText[0] = EnlargementText[0] = '\0';
   if  (settings_term_current.landscape   == SW_ONOFF_ON)                                     canvas_EPSLandscapify(x, LandscapifyText);
@@ -712,11 +720,29 @@ void canvas_EPSWrite(EPSComm *x)
    }
   if (chdir(settings_session_default.cwd) < 0) { ppl_fatal(__FILE__,__LINE__,"chdir into cwd failed."); }
 
+  // setlinejoin to avoid spikey wiggles
+  // fprintf(epsout, "0 setlinejoin\n");
+
   // Output macros which PyXPlot needs
   if (settings_term_current.TermType == SW_TERMTYPE_PS) fprintf(epsout, "%s", PS_PROLOG_TEXT);
   fprintf(epsout, "%s", EPS_PROLOG_TEXT);
   fprintf(epsout, "/ps { 1 } def\n/ps75 { ps .75 mul } def\n"); // Pointsize variables
-  if (x->PointTypesUsed[2]) { x->PointTypesUsed[0]=x->PointTypesUsed[1]=1; } // pt3 depends upon pt1 and pt2
+
+  if (x->PointTypesUsed[28]) { x->PointTypesUsed[14]=x->PointTypesUsed[ 0]=1; } // pt6 depends upon pt4 and pt5
+  if (x->PointTypesUsed[29]) { x->PointTypesUsed[15]=x->PointTypesUsed[ 1]=1; } // pt9 depends upon pt7 and pt8
+  if (x->PointTypesUsed[30]) { x->PointTypesUsed[16]=x->PointTypesUsed[ 2]=1; } // pt12 depends upon pt10 and pt11
+  if (x->PointTypesUsed[31]) { x->PointTypesUsed[17]=x->PointTypesUsed[ 3]=1; } // pt15 depends upon pt13 and pt14
+  if (x->PointTypesUsed[32]) { x->PointTypesUsed[18]=x->PointTypesUsed[ 4]=1; } // pt18 depends upon pt16 and pt17
+  if (x->PointTypesUsed[33]) { x->PointTypesUsed[19]=x->PointTypesUsed[ 5]=1; } // pt21 depends upon pt19 and pt20
+  if (x->PointTypesUsed[34]) { x->PointTypesUsed[20]=x->PointTypesUsed[ 6]=1; } // pt24 depends upon pt22 and pt23
+  if (x->PointTypesUsed[35]) { x->PointTypesUsed[21]=x->PointTypesUsed[ 7]=1; } // pt27 depends upon pt25 and pt26
+  if (x->PointTypesUsed[36]) { x->PointTypesUsed[22]=x->PointTypesUsed[ 8]=1; } // pt30 depends upon pt28 and pt29
+  if (x->PointTypesUsed[37]) { x->PointTypesUsed[23]=x->PointTypesUsed[ 9]=1; } // pt33 depends upon pt31 and pt32
+  if (x->PointTypesUsed[38]) { x->PointTypesUsed[24]=x->PointTypesUsed[10]=1; } // pt36 depends upon pt34 and pt35
+  if (x->PointTypesUsed[39]) { x->PointTypesUsed[25]=x->PointTypesUsed[11]=1; } // pt33 depends upon pt37 and pt38
+  if (x->PointTypesUsed[40]) { x->PointTypesUsed[26]=x->PointTypesUsed[12]=1; } // pt3 depends upon pt1 and pt2
+  if (x->PointTypesUsed[41]) { x->PointTypesUsed[27]=x->PointTypesUsed[13]=1; } // pt6 depends upon pt4 and pt5
+
   if (x->StarTypesUsed [6]) { x->StarTypesUsed [5]=1; } // st7 depends upon st6
   for (i=0; i<N_POINTTYPES; i++) if (x->PointTypesUsed[i]) fprintf(epsout, "%s\n", eps_PointTypes[i]);
   for (i=0; i<N_STARTYPES ; i++) if (x->StarTypesUsed [i]) { fprintf(epsout, "%s\n", eps_StarCore); break; }
@@ -765,17 +791,25 @@ void canvas_EPSWrite(EPSComm *x)
  }
 
 // Write a text item out from dvi buffer
-void canvas_EPSRenderTextItem(EPSComm *x, int pageno, double xpos, double ypos, int halign, int valign, char *colstr, double fontsize, double rotate, double *width, double *height)
+void canvas_EPSRenderTextItem(EPSComm *x, char **strout, int pageno, double xpos, double ypos, int halign, int valign, char *colstr, double fontsize, double rotate, double *width, double *height)
  {
   postscriptPage *dviPage;
   ListIterator *ListIter;
-  char *cptr;
+  char *cptr, *out;
+  static char *buffer=NULL;
+  const long   BUFLEN=131072;
+  long         bufpos=0;
   double bb_left, bb_right, bb_top, bb_bottom, xanchor, yanchor;
   double ab_left, ab_right, ab_top, ab_bottom; // The align box is different from the bounding box in that it gives the baseline height of the lowest line of text and the cap height of the highest line of text
 
   // If colstr is blank, we are painting with null ink
   if (colstr[0]=='\0') return;
 
+  // Write output to text buffer initially
+  if (buffer==NULL) buffer = lt_malloc_incontext(BUFLEN,0);
+  if (buffer==NULL) return;
+  out=buffer;
+
   if (x->dvi == NULL) { ppl_error(ERR_INTERNAL, -1, -1, "Attempting to display a text item before latex has generated it"); return; }
 
   // Fetch requested page of postscript
@@ -806,21 +840,21 @@ void canvas_EPSRenderTextItem(EPSComm *x, int pageno, double xpos, double ypos,
 
   // Update bounding box of canvas. For this, use BOUNDING box, not ALIGNMENT box.
   eps_core_PlotBoundingBox(x, xpos*M_TO_PS + (bb_left  - xanchor)*fontsize*cos(rotate) + (bb_bottom - yanchor)*fontsize*-sin(rotate),
-                              ypos*M_TO_PS + (bb_left  - xanchor)*fontsize*sin(rotate) + (bb_bottom - yanchor)*fontsize* cos(rotate), 0);
+                              ypos*M_TO_PS + (bb_left  - xanchor)*fontsize*sin(rotate) + (bb_bottom - yanchor)*fontsize* cos(rotate), 0, 1);
   eps_core_PlotBoundingBox(x, xpos*M_TO_PS + (bb_right - xanchor)*fontsize*cos(rotate) + (bb_bottom - yanchor)*fontsize*-sin(rotate),
-                              ypos*M_TO_PS + (bb_right - xanchor)*fontsize*sin(rotate) + (bb_bottom - yanchor)*fontsize* cos(rotate), 0);
+                              ypos*M_TO_PS + (bb_right - xanchor)*fontsize*sin(rotate) + (bb_bottom - yanchor)*fontsize* cos(rotate), 0, 1);
   eps_core_PlotBoundingBox(x, xpos*M_TO_PS + (bb_left  - xanchor)*fontsize*cos(rotate) + (bb_top    - yanchor)*fontsize*-sin(rotate),
-                              ypos*M_TO_PS + (bb_left  - xanchor)*fontsize*sin(rotate) + (bb_top    - yanchor)*fontsize* cos(rotate), 0);
+                              ypos*M_TO_PS + (bb_left  - xanchor)*fontsize*sin(rotate) + (bb_top    - yanchor)*fontsize* cos(rotate), 0, 1);
   eps_core_PlotBoundingBox(x, xpos*M_TO_PS + (bb_right - xanchor)*fontsize*cos(rotate) + (bb_top    - yanchor)*fontsize*-sin(rotate),
-                              ypos*M_TO_PS + (bb_right - xanchor)*fontsize*sin(rotate) + (bb_top    - yanchor)*fontsize* cos(rotate), 0);
+                              ypos*M_TO_PS + (bb_right - xanchor)*fontsize*sin(rotate) + (bb_top    - yanchor)*fontsize* cos(rotate), 0, 1);
 
 
   // Perform translation such that postscript text appears in the right place on the page
-  fprintf(x->epsbuffer, "gsave\n");
-  fprintf(x->epsbuffer, "%.2f %.2f translate\n", xpos * M_TO_PS, ypos * M_TO_PS);
-  fprintf(x->epsbuffer, "%.2f rotate\n", rotate * 180 / M_PI);
-  fprintf(x->epsbuffer, "%f %f scale\n", fontsize, fontsize);
-  fprintf(x->epsbuffer, "%.2f %.2f translate\n", -xanchor, -yanchor);
+  snprintf(out+bufpos, BUFLEN-bufpos, "gsave\n"); bufpos+=strlen(out+bufpos);
+  snprintf(out+bufpos, BUFLEN-bufpos, "%.2f %.2f translate\n", xpos * M_TO_PS, ypos * M_TO_PS); bufpos+=strlen(out+bufpos);
+  snprintf(out+bufpos, BUFLEN-bufpos, "%.2f rotate\n", rotate * 180 / M_PI); bufpos+=strlen(out+bufpos);
+  snprintf(out+bufpos, BUFLEN-bufpos, "%f %f scale\n", fontsize, fontsize); bufpos+=strlen(out+bufpos);
+  snprintf(out+bufpos, BUFLEN-bufpos, "%.2f %.2f translate\n", -xanchor, -yanchor); bufpos+=strlen(out+bufpos);
 
   // Copy postscript description of page out of dvi buffer
   ListIter = ListIterateInit(dviPage->text);
@@ -829,15 +863,20 @@ void canvas_EPSRenderTextItem(EPSComm *x, int pageno, double xpos, double ypos,
     cptr = (char *)ListIter->data;
     while (*cptr!='\0')
      {
-      if (*cptr!='\x01') fprintf(x->epsbuffer, "%c", *cptr);
-      else               fprintf(x->epsbuffer, "%s", colstr); // ASCII x01 is a magic code to tell us to revert to default colour
+      if (*cptr!='\x01') snprintf(out+bufpos, BUFLEN-bufpos, "%c", *cptr);
+      else               snprintf(out+bufpos, BUFLEN-bufpos, "%s", colstr); // ASCII x01 is a magic code to tell us to revert to default colour
+      bufpos+=strlen(out+bufpos);
       cptr++;
      }
     ListIter = ListIterate(ListIter, NULL);
    }
 
   // Undo translation and we're finished
-  fprintf(x->epsbuffer, "grestore\n");
+  snprintf(out+bufpos, BUFLEN-bufpos, "grestore\n"); bufpos+=strlen(out+bufpos);
+
+  // Either return string, or write output to file
+  if (strout==NULL) fprintf(x->epsbuffer, "%s", buffer);
+  else              *strout=buffer;
   return;
  }
 
@@ -869,6 +908,7 @@ void canvas_EPSEnlarge(EPSComm *x, char *transform)
   // Calculate dimensions of EPS image
   EPSwidth  = x->bb_right - x->bb_left;
   EPSheight = x->bb_top   - x->bb_bottom;
+  if ((EPSwidth<=0.0)||(EPSheight<=0.0)) return; // Don't attempt to enlarge pages of size zero
 
   // Work out some sensible margins for page
   margin_left = PAGEwidth / 10;
@@ -881,6 +921,7 @@ void canvas_EPSEnlarge(EPSComm *x, char *transform)
   // Work out what scaling factors will make page fit
   scaling_x = (PAGEwidth  - margin_left - margin_right ) / EPSwidth  * M_TO_PS;
   scaling_y = (PAGEheight - margin_top  - margin_bottom) / EPSheight * M_TO_PS;
+  if ((!gsl_finite(scaling_x))||(!gsl_finite(scaling_y))) return; // Don't attempt to enlarge pages of size zero
   if (scaling_y < scaling_x) scaling_x = scaling_y; // Lowest scaling factor is biggest enlargement we can do
 
   // Work out how to translate origin to (0,0)
