@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: ppl_canvasdraw.h 524 2009-12-28 18:29:06Z dcf21 $
+// $Id: ppl_canvasdraw.h 875 2010-07-08 15:44:28Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -28,7 +28,7 @@ void canvas_draw(unsigned char *unsuccessful_ops);
 void canvas_CallLaTeX(EPSComm *x);
 void canvas_MakeEPSBuffer(EPSComm *x);
 void canvas_EPSWrite(EPSComm *x);
-void canvas_EPSRenderTextItem(EPSComm *x, int pageno, double xpos, double ypos, int halign, int valign, char *colstr, double fontsize, double rotate, double *width, double *height);
+void canvas_EPSRenderTextItem(EPSComm *x, char **strout, int pageno, double xpos, double ypos, int halign, int valign, char *colstr, double fontsize, double rotate, double *width, double *height);
 void canvas_EPSLandscapify(EPSComm *x, char *transform);
 void canvas_EPSEnlarge(EPSComm *x, char *transform);
 
