@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: ppl_datafile.c 747 2010-05-03 19:48:51Z dcf21 $
+// $Id: ppl_datafile.c 936 2010-08-27 19:38:12Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -576,6 +576,11 @@ void DataFile_RotateRawData(RawDataTable **in, DataTable *out, char **UsingItems
   int           ContextOut = ( out)->MemoryContext;
 
   long int      file_linenumber[MAX_DATACOLS];
+  unsigned char hadspace[MAX_DATACOLS];
+  unsigned char hadcomma[MAX_DATACOLS];
+
+  for (i=0; i<MAX_DATACOLS; i++) hadspace[i] = 1;
+  for (i=0; i<MAX_DATACOLS; i++) hadcomma[i] = 0;
 
   while (1)
    {
@@ -586,19 +591,34 @@ void DataFile_RotateRawData(RawDataTable **in, DataTable *out, char **UsingItems
       for (i=0; ((i<blk->BlockPosition)&&(ItemsOnLine<MAX_DATACOLS)); i++)
        {
         if (blk->text[i]==NULL)
-         { RowData[ItemsOnLine++]=" "; }
+         {
+          RowData[ItemsOnLine]=" ";
+          file_linenumber[ItemsOnLine]=-1;
+          ItemsOnLine++;
+         }
         else
          {
-          while ((blk->text[i][0]!='\0') && (blk->text[i][0]<=' ')) blk->text[i]++;
-          if (blk->text[i][0]=='\0') RowData[ItemsOnLine++]=" ";
+          unsigned char caught=0;
+          for ( ; ((blk->text[i][0]!='\0') && (!caught)) ; blk->text[i]++)
+           {
+            if      (blk->text[i][0]<=' ') { hadspace[ItemsOnLine] = 1; }
+            else if (blk->text[i][0]==',') { blk->text[i]++; while ((blk->text[i][0]!='\0')&&(blk->text[i][0]<=' ')) { blk->text[i]++; } caught=1; hadspace[ItemsOnLine] = hadcomma[ItemsOnLine] = 1; }
+            else                           { if (hadspace[ItemsOnLine] && !hadcomma[ItemsOnLine]) { caught=1; } hadspace[ItemsOnLine] = hadcomma[ItemsOnLine] = 0; }
+            if (caught) break;
+           }
+
+          if (!caught)
+           {
+            RowData[ItemsOnLine]=" ";
+            file_linenumber[ItemsOnLine]=-1;
+            ItemsOnLine++;
+           }
           else
            {
             RowData        [ItemsOnLine] = blk->text    [i];
             file_linenumber[ItemsOnLine] = blk->FileLine[i];
             ItemsOnLine++;
             GotData = 1;
-            while  (blk->text[i][0]> ' ' )                            blk->text[i]++;
-            while ((blk->text[i][0]!='\0') && (blk->text[i][0]<=' ')) blk->text[i]++;
            }
          }
        }
@@ -618,16 +638,16 @@ void DataFile_RotateRawData(RawDataTable **in, DataTable *out, char **UsingItems
 // DataFile_read is the main entry point for reading a table of data from a data file
 // ----------------------------------------------------------------------------------
 
-void DataFile_read(DataTable **output, int *status, char *errout, char *filename, int index, int UsingRowCol, List *UsingList, List *EveryList, char *LabelStr, int Ncolumns, char *SelectCriterion, int continuity, char *SortBy, int SortByContinuity, int *ErrCounter)
+void DataFile_read(DataTable **output, int *status, char *errout, char *filename, int index, int UsingRowCol, List *UsingList, unsigned char AutoUsingList, List *EveryList, char *LabelStr, int Ncolumns, char *SelectCriterion, int continuity, char *SortBy, int SortByContinuity, int *ErrCounter)
  {
-  unsigned char AutoUsingList=0, HadNonNullUsingItem=0, ReadFromCommandLine=0, discontinuity=0, hadwhitespace, hadcomma, OneColumnInput=1;
+  unsigned char HadNonNullUsingItem=0, ReadFromCommandLine=0, discontinuity=0, hadwhitespace, hadcomma, OneColumnInput=1;
   int           UsingLen, logi, logj, ContextOutput, ContextRough, ContextRaw;
   char         *UsingItems[USING_ITEMS_MAX], LineNumberStr[32];
   ListIterator *listiter;
   Dict         *tempdict;
   int           linestep=1, blockstep=1, linefirst=-1, blockfirst=-1, linelast=-1, blocklast=-1;
   long int      index_number, linenumber_count, linenumber_stepcnt, block_count, block_stepcnt, prev_blanklines, file_linenumber, ItemsOnLine;
-  FILE         *filtered_input;
+  FILE         *filtered_input=NULL;
   char          linebuffer[LSTR_LENGTH], *lineptr, *cptr;
 
   int i, j, k, l, m, *intptr;
@@ -978,7 +998,7 @@ void DataFile_read(DataTable **output, int *status, char *errout, char *filename
 
       // Add line numbers as first column to one-column datafiles
       if  (ItemsOnLine >  1) OneColumnInput=0;
-      if ((ItemsOnLine == 1) && (OneColumnInput))
+      if ((ItemsOnLine == 1) && AutoUsingList && OneColumnInput)
        {
         ColumnData[ItemsOnLine++]=ColumnData[0];
         ColumnData[0]=LineNumberStr;
@@ -1018,18 +1038,21 @@ void DataFile_read(DataTable **output, int *status, char *errout, char *filename
 
 #define COUNTEDERR2F if (*ErrCounter==0) { sprintf(temp_err_string, "%s: Too many errors: no more errors will be shown.",buffer); ppl_warning(ERR_STACKED, temp_err_string); } }
 
-void DataFile_FromFunctions(double *OrdinateRaster, unsigned char FlagParametric, int RasterLen, value *RasterUnits, DataTable **output, int *status, char *errout, char **fnlist, int fnlist_len, List *UsingList, char *LabelStr, int Ncolumns, char *SelectCriterion, int continuity, char *SortBy, int SortByContinuity, int *ErrCounter)
+void DataFile_FromFunctions(double *OrdinateRaster, unsigned char FlagParametric, int RasterLen, value *RasterUnits, double *OrdinateYRaster, int RasterYLen, value *RasterYUnits, DataTable **output, int *status, char *errout, char **fnlist, int fnlist_len, List *UsingList, unsigned char AutoUsingList, char *LabelStr, int Ncolumns, char *SelectCriterion, int continuity, char *SortBy, int SortByContinuity, int *ErrCounter)
  {
-  unsigned char AutoUsingList=0, HadNonNullUsingItem=0, discontinuity=0;
-  int           UsingLen, logi, logj, i, j, k, ContextOutput;
+  unsigned char HadNonNullUsingItem=0, discontinuity=0, SampleGrid;
+  int           UsingLen, logi, logj, a, i, j, k, ContextOutput;
+  long          ExpectedNrows;
   char         *UsingItems[USING_ITEMS_MAX], buffer[FNAME_LENGTH];
   value         ColumnData_val[USING_ITEMS_MAX+2];
   ListIterator *listiter;
   Dict         *tempdict;
-  value        *OrdinateVar, DummyTemp;
+  value        *OrdinateVar , DummyTemp ;
+  value        *OrdinateVar2, DummyTemp2;
 
   // Init
   if (DEBUG) { sprintf(temp_err_string, "Evaluated supplied set of functions."); ppl_log(temp_err_string); }
+  SampleGrid = (OrdinateYRaster != NULL);
 
   // Read items out of Using List into an array of strings
   UsingLen=0; listiter = ListIterateInit(UsingList);
@@ -1066,7 +1089,8 @@ void DataFile_FromFunctions(double *OrdinateRaster, unsigned char FlagParametric
   else if ((UsingLen==1) && (Ncolumns==2)) // Prepend ordinate value if only one number specified in using statement
    {
     UsingItems[1] = UsingItems[0];
-    UsingItems[0] = "0";
+    if ((UsingItems[0] = (char *)lt_malloc(10))==NULL) { sprintf(errout,"Out of memory."); *status=1; if (DEBUG) ppl_log(errout); return; };
+    sprintf(UsingItems[0], "0");
     UsingLen++;
    }
 
@@ -1087,7 +1111,7 @@ void DataFile_FromFunctions(double *OrdinateRaster, unsigned char FlagParametric
   if (UsingLen != Ncolumns) { sprintf(errout,"The supplied using ... clause contains the wrong number of items. We need %d columns of data, but %d have been supplied.", Ncolumns, UsingLen); *status=1; if (DEBUG) ppl_log(errout); return; }
 
   // If SortBy is not NULL, add it to using list
-  if (SortBy!=NULL)
+  if ((!SampleGrid) && (SortBy!=NULL))
    {
     if (strcmp(SortBy, "@")==0) UsingItems[UsingLen++] = UsingItems[0];
     else                        UsingItems[UsingLen++] = SortBy;
@@ -1095,57 +1119,80 @@ void DataFile_FromFunctions(double *OrdinateRaster, unsigned char FlagParametric
 
   // Keep a record of the memory context we're going to output into
   ContextOutput = lt_GetMemContext();
-  *output = DataFile_NewDataTable(UsingLen, ContextOutput, RasterLen);
+  *output = DataFile_NewDataTable(UsingLen, ContextOutput, SampleGrid ? (RasterLen*RasterYLen) : RasterLen);
   if (*output == NULL) { strcpy(errout, "Out of memory whilst trying to allocate data table to read data from file."); *status=1; if (DEBUG) ppl_log(errout); return; }
 
   // Get a pointer to the value of the variable 'x' in the user's variable space
-  DictLookup(_ppl_UserSpace_Vars, FlagParametric?"t":"x", NULL, (void **)&OrdinateVar);
-  if (OrdinateVar!=NULL)
+  OrdinateVar2 = NULL;
+  for (a=0; a<=SampleGrid; a++)
    {
-    DummyTemp = *OrdinateVar;
-   }
-  else // If variable is not defined, create it now
-   {
-    ppl_units_zero(&DummyTemp);
-    DictAppendValue(_ppl_UserSpace_Vars, FlagParametric?"t":"x", DummyTemp);
-    DictLookup(_ppl_UserSpace_Vars, FlagParametric?"t":"x", NULL, (void **)&OrdinateVar);
-    DummyTemp.modified = 2;
-   }
-  *OrdinateVar = *RasterUnits;
-  OrdinateVar->imag        = 0.0;
-  OrdinateVar->FlagComplex = 0;
+    char *v; value **_OrdinateVar , *_DummyTemp, **_RasterUnits;
+    if (!a) { v="x"; _OrdinateVar=&OrdinateVar ; _DummyTemp=&DummyTemp ; _RasterUnits=&RasterUnits ; }
+    else    { v="y"; _OrdinateVar=&OrdinateVar2; _DummyTemp=&DummyTemp2; _RasterUnits=&RasterYUnits; }
 
+    DictLookup(_ppl_UserSpace_Vars, FlagParametric?"t":v, NULL, (void **)_OrdinateVar);
+    if (*_OrdinateVar!=NULL)
+     {
+      *_DummyTemp = **_OrdinateVar;
+     }
+    else // If variable is not defined, create it now
+     {
+      ppl_units_zero(_DummyTemp);
+      DictAppendValue(_ppl_UserSpace_Vars, FlagParametric?"t":v, *_DummyTemp);
+      DictLookup(_ppl_UserSpace_Vars, FlagParametric?"t":v, NULL, (void **)_OrdinateVar);
+      (*_DummyTemp).modified = 2;
+     }
+    **_OrdinateVar = **_RasterUnits;
+    (*_OrdinateVar)->imag        = 0.0;
+    (*_OrdinateVar)->FlagComplex = 0;
+   }
 
   // Loop over ordinate values
-  for (i=0; i<RasterLen; i++)
+  {
+   int i, ilen = RasterLen, i2, i2len = SampleGrid ? RasterYLen : 1; long p=0;
+   for (i2=0; i2<i2len; i2++) for (i=0; i<ilen; i++, p++)
    {
     OrdinateVar->real = OrdinateRaster[i];
-    sprintf(buffer, "%c=%s", (FlagParametric?'t':'x'), NumericDisplay(OrdinateVar->real, 0, settings_term_current.SignificantFigures, 0));
+    if (SampleGrid) OrdinateVar2->real = OrdinateYRaster[i2];
+    if (SampleGrid) sprintf(buffer, "x=%s; y=%s", NumericDisplay(OrdinateVar->real, 0, settings_term_current.SignificantFigures, 0), NumericDisplay(OrdinateVar2->real, 1, settings_term_current.SignificantFigures, 0));
+    else            sprintf(buffer, "%c=%s", (FlagParametric?'t':'x'), NumericDisplay(OrdinateVar->real, 0, settings_term_current.SignificantFigures, 0));
     ppl_units_zero(ColumnData_val+0);
-    ColumnData_val[0].real = i;
+    ColumnData_val[0].real = p;
     if (!FlagParametric) ColumnData_val[1] = *OrdinateVar;
+    if (SampleGrid     ) ColumnData_val[2] = *OrdinateVar2;
     for (j=0; j<fnlist_len; j++)
      {
       *status=-1; k=-1;
-      ppl_EvaluateAlgebra(fnlist[j], ColumnData_val+j+1+(!FlagParametric), 0, &k, 0, status, errout, 0);
+      ppl_EvaluateAlgebra(fnlist[j], ColumnData_val+j+1+(!FlagParametric)+SampleGrid, 0, &k, 0, status, errout, 0);
       if (k<strlen(fnlist[j])) { sprintf(errout, "Expression '%s' is not syntactically valid %d %ld", fnlist[j],k,(long)strlen(fnlist[j])); *status=1; if (DEBUG) ppl_log(errout); return; }
       if (*status>=0) { COUNTEDERR1; sprintf(temp_err_string, "%s: Could not evaluate expression <%s>. The error, encountered at character position %d, was: '%s'", buffer, fnlist[j], *status, errout); ppl_error(ERR_NUMERIC, -1, -1, temp_err_string); COUNTEDERR2F; *status=1; break; }
       else            { *status=0; }
      }
+    ExpectedNrows = (*output)->Nrows+1;
     if (!(*status))
      {
-      DataFile_ApplyUsingList(*output, ContextOutput, NULL, ColumnData_val, fnlist_len+(!FlagParametric), UsingItems, UsingLen, buffer, 0, NULL, i, 0, 0, DATAFILE_COL, "column", NULL, 0, NULL, 0, LabelStr, SelectCriterion, continuity, &discontinuity, ErrCounter, status, errout);
+      DataFile_ApplyUsingList(*output, ContextOutput, NULL, ColumnData_val, fnlist_len+(!FlagParametric)+SampleGrid, UsingItems, UsingLen, buffer, 0, NULL, i, 0, 0, DATAFILE_COL, "column", NULL, 0, NULL, 0, LabelStr, SelectCriterion, continuity, &discontinuity, ErrCounter, status, errout);
      } else {
-      discontinuity = 1;
+      if (!SampleGrid) discontinuity = 1;
+     }
+    if (SampleGrid && ((*output)->Nrows<ExpectedNrows)) // If sampling a grid, MUST have entry for every point
+     {
+      int i;
+      for (i=0; i<(*output)->Ncolumns; i++)
+       (*output)->current->data_real[i + (*output)->current->BlockPosition * (*output)->Ncolumns].d = GSL_NAN;
+      (*output)->current->text[(*output)->current->BlockPosition] = NULL;
+      DataFile_DataTable_AddRow(*output);
      }
     *status=0;
    }
+  }
 
-  // Reset the variable x to its old value
+  // Reset the variable x (and maybe y or t) to its old value
   *OrdinateVar = DummyTemp;
+  if (OrdinateVar2!=NULL) *OrdinateVar2 = DummyTemp2;
 
   // If data is to be sorted, sort it now
-  if (SortBy != NULL)
+  if ((!SampleGrid) && (SortBy != NULL))
    {
     *output = DataTable_sort(*output, UsingLen-1, SortByContinuity==DATAFILE_CONTINUOUS);
    }
@@ -1185,6 +1232,8 @@ DataTable *DataTable_sort(DataTable *in, int SortColumn, int IgnoreContinuity)
   DataTable_sorter *sorter;
   DataTable        *output;
 
+  if (in==NULL) return NULL;
+
   Nc = in->Ncolumns;
   Nco= Nc-1;
   Nr = in->Nrows;
