@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: ppl_fft.c 745 2010-05-03 18:06:33Z dcf21 $
+// $Id: ppl_fft.c 866 2010-07-03 15:40:57Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -69,7 +69,7 @@ int directive_fft(Dict *command)
   double        TempDbl, pos[USING_ITEMS_MAX], norm;
   value        *min[USING_ITEMS_MAX], *max[USING_ITEMS_MAX], *step[USING_ITEMS_MAX], x, FirstVal;
   unsigned char inverse;
-  char         *cptr, *tempstr, *filename, *outfunc, *infunc, *scratchpad, *errtext, *SelectCrit=NULL;
+  char         *cptr, *tempstr, *filename=NULL, *outfunc, *infunc, *scratchpad, *errtext, *SelectCrit=NULL;
   void (*WindowType)(value *,int,int *,int *);
   List         *RangeList=NULL, *UsingList=NULL, *EveryList=NULL;
   ListIterator *ListIter;
@@ -171,7 +171,7 @@ int directive_fft(Dict *command)
     // Read data from file
     status=0;
     errtext = (char *)lt_malloc(LSTR_LENGTH);
-    DataFile_read(&data, &status, errtext, filename, *indexptr, rowcol, UsingList, EveryList, NULL, Ndims+2, SelectCrit, DATAFILE_CONTINUOUS, NULL, -1, &ErrCount);
+    DataFile_read(&data, &status, errtext, filename, *indexptr, rowcol, UsingList, 0, EveryList, NULL, Ndims+2, SelectCrit, DATAFILE_CONTINUOUS, NULL, -1, &ErrCount);
     if (status) { ppl_error(ERR_GENERAL, -1, -1, errtext); return 1; }
     if (data->Nrows==0) { ppl_error(ERR_FILE, -1, -1, "No data was read from file"); return 1; }
 
