@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: ppl_fit.c 745 2010-05-03 18:06:33Z dcf21 $
+// $Id: ppl_fit.c 953 2010-09-09 16:11:57Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -412,7 +412,7 @@ int directive_fit(Dict *command)
   ContextDataTab = lt_DescendIntoNewContext();
 
   // Read data from file
-  DataFile_read(&data, &status, errtext, filename, *indexptr, rowcol, UsingList, EveryList, NULL, NExpect, SelectCrit, DATAFILE_CONTINUOUS, NULL, -1, &ErrCount);
+  DataFile_read(&data, &status, errtext, filename, *indexptr, rowcol, UsingList, 0, EveryList, NULL, NExpect, SelectCrit, DATAFILE_CONTINUOUS, NULL, -1, &ErrCount);
   if (status) { ppl_error(ERR_GENERAL, -1, -1, errtext); return 1; }
 
   // Check that the FirstEntries above have the same units as any supplied ranges
@@ -521,6 +521,8 @@ int directive_fit(Dict *command)
   // Estimate the size of the errorbars on the supplied data if no errorbars were supplied (this doesn't affect best fit position, but does affect error estimates)
   if (!DataComm.FlagYErrorBars)
    {
+    sprintf(temp_err_string, "\n# Estimating the size of the error bars on supplied data.\n# This may take a while.\n# The fit command can be made to run very substantially faster if the 'withouterrors' option is set.");
+    ppl_report(temp_err_string);
     status = FitMinimiseIterate(&DataComm, &FitSigmaData, 1);
     if (status) { ppl_error(ERR_GENERAL, -1, -1, DataComm.errtext); gsl_vector_free(BestFitParamVals); return 1; }
     FirstVals[NArgs].real = DataComm.SigmaData;
