@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: ppl_flowctrl.c 763 2010-05-13 16:37:14Z dcf21 $
+// $Id: ppl_flowctrl.c 940 2010-08-31 16:59:32Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -181,7 +181,8 @@ int directive_do(Dict *command, int IterLevel)
   else if (status == -2) { ppl_error(ERR_SYNTAX, -1, -1, "do clause should be terminated with a while statement."); return 1; }
 
   // Check that final line has a while clause on it
-  cmd2 = parse(cptr);
+  cmd2 = parse(cptr, IterLevel);
+
   if (cmd2 == NULL) return 1; // Parser has already thrown an error, we assume
   DictLookup(cmd2,"directive",NULL,(void **)(&cptr));
   if (strcmp(cptr,"while")!=0) { ppl_error(ERR_SYNTAX, -1, -1, "Only the statement 'while' can be placed after a } here."); return 1; }
@@ -278,7 +279,7 @@ int directive_for(Dict *command, int IterLevel)
   value     *iterval;
   double     itervalD;
   value      step_dummy;
-  unsigned char backwards;
+  unsigned char backwards_step;
   char      *cptr;
   cmd_chain  chain     = NULL;
   cmd_chain  chainiter = NULL;
@@ -305,10 +306,10 @@ int directive_for(Dict *command, int IterLevel)
   if (!ppl_units_DimEqual(start, end)) { sprintf(temp_err_string, "The start and end values in this for loop are not dimensionally compatible. The start value has units of <%s>, while the end value has units of <%s>.", ppl_units_GetUnitStr(start, NULL, NULL, 0, 1, 0), ppl_units_GetUnitStr(end, NULL, NULL, 1, 1, 0)); ppl_error(ERR_NUMERIC, -1, -1, temp_err_string); return 1; }
   if (!ppl_units_DimEqual(start, step)) { sprintf(temp_err_string, "The start value and step size  in this for loop are not dimensionally compatible. The start value has units of <%s>, while the step size has units of <%s>.", ppl_units_GetUnitStr(start, NULL, NULL, 0, 1, 0), ppl_units_GetUnitStr(step, NULL, NULL, 1, 1, 0)); ppl_error(ERR_NUMERIC, -1, -1, temp_err_string); return 1; }
 
-  if (start->real < end->real) backwards=0;
-  else                         backwards=1;
+  backwards_step  = (step->real < 0);
 
-  if (((!backwards) && (step->real<=0)) || ((backwards) && (step->real>=0))) { sprintf(temp_err_string, "The projected number of steps in this for loop is infinite."); ppl_error(ERR_NUMERIC, -1, -1, temp_err_string); return 1; }
+  if ((step->real==0.0) || ((backwards_step)&&(start->real < end->real)) || ((!backwards_step)&&(start->real > end->real)))
+   { sprintf(temp_err_string, "The projected number of steps in this for loop is infinite."); ppl_error(ERR_NUMERIC, -1, -1, temp_err_string); return 1; }
 
   // Fetch lines and add them into loop chain until we get a }
   while (status==0)
@@ -332,7 +333,7 @@ int directive_for(Dict *command, int IterLevel)
 
   // Execute this for loop repeatedly
   status = 0;
-  while (((!backwards)&&(iterval->real < end->real)) || ((backwards)&&(iterval->real > end->real)))
+  while (((!backwards_step)&&(iterval->real <= end->real)) || ((backwards_step)&&(iterval->real >= end->real)))
    {
     chainiter = chain;
     status = loop_execute(&chainiter, 1, 0, IterLevel);
@@ -532,7 +533,7 @@ void directive_foreach_LoopOverData(Dict *command, char *filename, cmd_chain *ch
    if (ListIter != NULL) { sprintf(temp_err_string, "Too many ranges supplied to the 'foreach ... in datafile' construct. %d ranges were supplied, even though only %ld variables are being read.", ListLen(RangeList), i); ppl_error(ERR_SYNTAX, -1, -1, temp_err_string); *status=1; return; }
 
   ContextDataTab = lt_DescendIntoNewContext();
-  DataFile_read(&data, status, errtext, filename, *indexptr, rowcol, UsingList, EveryList, NULL, i, SelectCrit, DATAFILE_CONTINUOUS, NULL, -1, &ErrCount);
+  DataFile_read(&data, status, errtext, filename, *indexptr, rowcol, UsingList, 0, EveryList, NULL, i, SelectCrit, DATAFILE_CONTINUOUS, NULL, -1, &ErrCount);
   if (*status) { ppl_error(ERR_GENERAL, -1, -1, errtext); *status=1; return; }
 
   // Check that the FirstEntries above have the same units as any supplied ranges
@@ -629,7 +630,7 @@ int directive_ifelse(Dict *command, int state, int IterLevel) // state = 0 (don'
   // See whether final line has an else clause on it
   if (status == -1)
    {
-    cmd2 = parse(cptr);
+    cmd2 = parse(cptr, IterLevel);
     if (cmd2 == NULL) return 1; // Parser has already thrown an error, we assume
     DictLookup(cmd2,"directive",NULL,(void **)(&cptr));
     if (strcmp(cptr,"else")!=0) { ppl_error(ERR_SYNTAX, -1, -1, "Only the statement 'else' can be placed after a } here."); return 1; }
@@ -757,6 +758,8 @@ int SubroutineCall(char *name, List *ArgList, char *errtext, int IterLevel)
   value        *VarData, *InputValue, *ValueBuffer, TempValue;
   cmd_chain     chainiter = NULL;
 
+  if (IterLevel > MAX_ITERLEVEL_DEPTH) { ppl_error(ERR_GENERAL, -1, -1, "Maximum recursion depth exceeded."); return 1; }
+
   ppl_units_zero(&TempValue);
 
   // Look up subroutine name
@@ -837,7 +840,7 @@ int directive_call(Dict *command, int IterLevel)
   List *ArgList;
   DictLookup(command,"subroutine_name",NULL,(void **)(&name));
   DictLookup(command,"argument_list,",NULL,(void **)(&ArgList));
-  return SubroutineCall(name, ArgList, NULL, IterLevel);
+  return SubroutineCall(name, ArgList, NULL, IterLevel+1);
  }
 
 int CallSubroutineFromAlgebra(char *FunctionName, char *ArgStart, int *endpos, int *errpos, char *errtext, int IterLevel)
@@ -891,7 +894,7 @@ int CallSubroutineFromAlgebra(char *FunctionName, char *ArgStart, int *endpos, i
 
   *errpos = -1;
   *endpos = i+1;
-  output = SubroutineCall(FunctionName, ArgList, errtext, IterLevel);
+  output = SubroutineCall(FunctionName, ArgList, errtext, IterLevel+1);
   return output;
  }
 
