@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: ppl_histogram.c 752 2010-05-07 17:13:47Z dcf21 $
+// $Id: ppl_histogram.c 886 2010-07-14 16:21:46Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -58,7 +58,7 @@ int directive_histogram(Dict *command)
   long int   i, j, k, Nrows;
   int        ContextOutput, ContextLocalVec, ContextDataTab, status=0, index=-1, *indexptr, rowcol=DATAFILE_COL, continuity, ErrCount=DATAFILE_NERRS;
   char       errtext[LSTR_LENGTH], *cptr, *filename=NULL, *histfunc=NULL, *tempstr=NULL, *SelectCrit=NULL;
-  double    *xdata, xbinmin, xbinmax, BinOriginDbl, BinWidthDbl;
+  double    *xdata, xbinmin, xbinmax, BinOriginDbl=0, BinWidthDbl;
   List      *UsingList=NULL, *EveryList=NULL, *templist;
   Dict      *tempdict;
   ListIterator *listiter;
@@ -98,7 +98,7 @@ int directive_histogram(Dict *command)
   ContextLocalVec= lt_DescendIntoNewContext();
   ContextDataTab = lt_DescendIntoNewContext();
 
-  DataFile_read(&data, &status, errtext, filename, *indexptr, rowcol, UsingList, EveryList, NULL, 1, SelectCrit, continuity, NULL, -1, &ErrCount);
+  DataFile_read(&data, &status, errtext, filename, *indexptr, rowcol, UsingList, 0, EveryList, NULL, 1, SelectCrit, continuity, NULL, -1, &ErrCount);
   if (status) { ppl_error(ERR_GENERAL, -1, -1, errtext); return 1; }
   if (data->Nrows<3) { ppl_error(ERR_NUMERIC, -1, -1, "Histogram construction is only possible on data sets with members at at least three values of x."); return 1; }
 
@@ -162,6 +162,7 @@ int directive_histogram(Dict *command)
   output = (HistogramDescriptor *)lt_malloc_incontext(sizeof(HistogramDescriptor), 0);
   if (output == NULL) { ppl_error(ERR_MEMORY, -1, -1,"Out of memory"); return 1; }
   output->unit     = FirstEntry;
+  output->log      = logaxis;
   output->filename = (char *)lt_malloc_incontext(strlen(filename)+1, 0);
   if (output->filename == NULL) { ppl_error(ERR_MEMORY, -1, -1,"Out of memory"); return 1; }
   strcpy(output->filename , filename);
@@ -307,7 +308,7 @@ int directive_histogram(Dict *command)
 
 void ppl_histogram_evaluate(char *FuncName, HistogramDescriptor *desc, value *in, value *out, int *status, char *errout)
  {
-  int i;
+  long   i, Nsteps, len, ss, pos;
   double dblin;
 
   if (!ppl_units_DimEqual(in, &desc->unit))
@@ -334,7 +335,18 @@ void ppl_histogram_evaluate(char *FuncName, HistogramDescriptor *desc, value *in
   for (i=0; i<UNITS_MAX_BASEUNITS; i++) if (out->exponent[i]!=0.0) out->exponent[i]*=-1; // Output has units of 'per x'
   if ((desc->Nbins<1) || (dblin<desc->bins[0]) || (dblin>desc->bins[desc->Nbins-1])) return; // Query is outside range of histogram
 
-  for (i=1; i<desc->Nbins; i++) if (desc->bins[i]>dblin) { out->real = desc->binvals[i-1]; return; }
+  len    = desc->Nbins;
+  Nsteps = (long)ceil(log(len)/log(2));
+  for (pos=i=0; i<Nsteps; i++)
+   {
+    ss = 1<<(Nsteps-1-i);
+    if (pos+ss>=len) continue;
+    if (desc->bins[pos+ss]<=dblin) pos+=ss;
+   }
+  if      (desc->bins[pos]>dblin) return; // Off left end
+  else if (pos==len-1)            return; // Off right end
+
+  out->real = desc->binvals[pos];
   return;
  }
 
