@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: ppl_interpolation.h 491 2009-12-17 20:36:30Z dcf21 $
+// $Id: ppl_interpolation.h 867 2010-07-03 16:59:09Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -27,14 +27,20 @@
 #include "ppl_units.h"
 #include "ppl_userspace.h"
 
-#define INTERP_LINEAR 23001
-#define INTERP_LOGLIN 23002
-#define INTERP_SPLINE 23003
-#define INTERP_AKIMA  23004
-#define INTERP_POLYN  23005
+#define INTERP_AKIMA    23001
+#define INTERP_LINEAR   23002
+#define INTERP_LOGLIN   23003
+#define INTERP_POLYN    23004
+#define INTERP_SPLINE   23005
+#define INTERP_STEPWISE 23006
+#define INTERP_2D       23007
+#define INTERP_BMPR     23008
+#define INTERP_BMPG     23009
+#define INTERP_BMPB     23010
 
 int  directive_interpolate(Dict *command, int mode);
 void ppl_spline_evaluate(char *FuncName, SplineDescriptor *desc, value *in, value *out, int *status, char *errout);
+void ppl_interp2d_evaluate(const char *FuncName, SplineDescriptor *desc, const value *in1, const value *in2, const unsigned char bmp, value *out, int *status, char *errout);
 
 #endif
 
