@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: ppl_parser.h 491 2009-12-17 20:36:30Z dcf21 $
+// $Id: ppl_parser.h 918 2010-08-02 12:06:57Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -26,7 +26,7 @@
 #include "ListTools/lt_dict.h"
 
 void   ppl_commands_read();
-Dict  *parse             (char *line);
+Dict  *parse             (char *line, int IterLevel);
 char **ppl_rl_completion (const char *text, int start, int end);
 
 #endif
