@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: ppl_readconfig.c 811 2010-05-25 15:17:48Z dcf21 $
+// $Id: ppl_readconfig.c 918 2010-08-02 12:06:57Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -164,9 +164,49 @@ void ReadConfigFile(char *ConfigFname)
       else if (strcmp(setkey, "CLIP"         )==0)
         if ((i=FetchSettingByName(setvalue,SW_ONOFF_INT, SW_ONOFF_STR ))>0)                      settings_graph_default.clip          = i;
         else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting Clip."         , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
+      else if (strcmp(setkey, "COLKEY"       )==0)
+        if ((i=FetchSettingByName(setvalue,SW_ONOFF_INT, SW_ONOFF_STR ))>0)                      settings_graph_default.ColKey        = i;
+        else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting ColKey."       , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
+      else if (strcmp(setkey, "COLKEYPOS"    )==0)
+        if ((i=FetchSettingByName(setvalue,SW_COLKEYPOS_INT,SW_COLKEYPOS_STR))>0)                settings_graph_default.ColKeyPos     = i;
+        else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting ColKeyPos."    , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
       else if (strcmp(setkey, "COLOUR"       )==0)
         if ((i=FetchSettingByName(setvalue,SW_ONOFF_INT, SW_ONOFF_STR ))>0)                      settings_term_default .colour        = i;
         else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting Colour."       , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
+      else if (strcmp(setkey, "CONTOURS"     )==0)
+        if  (fl=GetFloat(setvalue, &i), ((gsl_finite(fl))&&(i==strlen(setvalue))))               settings_graph_default.ContoursN     = max((int)fl, 2);
+        else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting Contours."     , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
+      else if (strcmp(setkey, "CONTOURS_LABEL")==0)
+        if ((i=FetchSettingByName(setvalue,SW_ONOFF_INT, SW_ONOFF_STR ))>0)                      settings_graph_default.ContoursLabel = i;
+        else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting Contours_Label.", linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
+
+
+#define DO_CRANGE(c,X) \
+      else if (strcmp(setkey, "C" X "RANGE_LOG"   )==0) \
+        if ((i=FetchSettingByName(setvalue,SW_BOOL_INT,SW_BOOL_STR))>0)                          settings_graph_default.Clog[c]            = i; \
+        else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting C" X "Range_Log."   , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; } \
+      else if (strcmp(setkey, "C" X "RANGE_MIN"   )==0) \
+        if  (fl=GetFloat(setvalue, &i), ((gsl_finite(fl))&&(i==strlen(setvalue))))               { settings_graph_default.Cmin[c].real = fl; settings_graph_default.Cminauto[c] = SW_BOOL_FALSE; } \
+        else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting C" X "Range_Min."   , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; } \
+      else if (strcmp(setkey, "C" X "RANGE_MIN_AUTO")==0) \
+        if  ((i=FetchSettingByName(setvalue,SW_BOOL_INT,SW_BOOL_STR))>0)                         settings_graph_default.Cminauto[c]        = i; \
+        else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting C" X "Range_Min_Auto.", linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; } \
+      else if (strcmp(setkey, "C" X "RANGE_MAX"   )==0) \
+        if  (fl=GetFloat(setvalue, &i), ((gsl_finite(fl))&&(i==strlen(setvalue))))               { settings_graph_default.Cmax[c].real = fl; settings_graph_default.Cmaxauto[c] = SW_BOOL_FALSE; } \
+        else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting C" X "Range_Max."   , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; } \
+      else if (strcmp(setkey, "C" X "RANGE_MAX_AUTO")==0) \
+        if  ((i=FetchSettingByName(setvalue,SW_BOOL_INT,SW_BOOL_STR))>0)                         settings_graph_default.Cmaxauto[c]        = i; \
+        else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting C" X "Range_Max_Auto.", linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; } \
+      else if (strcmp(setkey, "C" X "RANGE_RENORM")==0) \
+        if  ((i=FetchSettingByName(setvalue,SW_BOOL_INT,SW_BOOL_STR))>0)                         settings_graph_default.Crenorm[c]         = i; \
+        else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting C" X "Range_Renorm.", linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; } \
+      else if (strcmp(setkey, "C" X "RANGE_REVERSE")==0) \
+        if  ((i=FetchSettingByName(setvalue,SW_BOOL_INT,SW_BOOL_STR))>0)                         settings_graph_default.Creverse[c]        = i; \
+        else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting C" X "Range_Reverse.", linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
+
+
+      DO_CRANGE(0,"1")  DO_CRANGE(1,"2")  DO_CRANGE(2,"3")  DO_CRANGE(3,"4")
+
       else if (strcmp(setkey, "DATASTYLE"    )==0)
         if ((i=FetchSettingByName(setvalue,SW_STYLE_INT, SW_STYLE_STR ))>0)                      settings_graph_default.DataStyle.style = i;
         else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting DataStyle."    , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
@@ -231,7 +271,7 @@ void ReadConfigFile(char *ConfigFname)
         if ((i=FetchSettingByName(setvalue,SW_ONOFF_INT, SW_ONOFF_STR ))>0) settings_term_default .landscape     = i;
         else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting Landscape."    , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
       else if (strcmp(setkey, "LINEWIDTH"    )==0)
-        if  (fl=GetFloat(setvalue, &i), ((gsl_finite(fl))&&(i==strlen(setvalue))))               settings_graph_default.LineWidth     = fl;
+        if  (fl=GetFloat(setvalue, &i), ((gsl_finite(fl))&&(i==strlen(setvalue))))               settings_graph_default.LineWidth     = fabs(fl);
         else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting LineWidth."    , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
       else if (strcmp(setkey, "MULTIPLOT"    )==0)
         if ((i=FetchSettingByName(setvalue,SW_ONOFF_INT, SW_ONOFF_STR ))>0)                      settings_term_default .multiplot     = i;
@@ -290,6 +330,21 @@ void ReadConfigFile(char *ConfigFname)
       else if (strcmp(setkey, "SAMPLES"      )==0)
         if  (fl=GetFloat(setvalue, &i), ((gsl_finite(fl))&&(i==strlen(setvalue))))               settings_graph_default.samples       = max((int)fl, 2);
         else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting Samples."      , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
+      else if (strcmp(setkey, "SAMPLES_METHOD")==0)
+        if  ((i=FetchSettingByName(setvalue,SW_SAMPLEMETHOD_INT,SW_SAMPLEMETHOD_STR))>0)         settings_graph_default.Sample2DMethod = i;
+        else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting Samples_Method." , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
+      else if (strcmp(setkey, "SAMPLES_X"    )==0)
+        if  (fl=GetFloat(setvalue, &i), ((gsl_finite(fl))&&(i==strlen(setvalue))))               { settings_graph_default.SamplesX = max((int)fl, 2); settings_graph_default.SamplesXAuto = SW_BOOL_FALSE; }
+        else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting Samples_X."      , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
+      else if (strcmp(setkey, "SAMPLES_X_AUTO")==0)
+        if  ((i=FetchSettingByName(setvalue,SW_BOOL_INT,SW_BOOL_STR))>0)                         settings_graph_default.SamplesXAuto  = i;
+        else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting Samples_X_Auto." , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
+      else if (strcmp(setkey, "SAMPLES_Y"    )==0)
+        if  (fl=GetFloat(setvalue, &i), ((gsl_finite(fl))&&(i==strlen(setvalue))))               { settings_graph_default.SamplesY = max((int)fl, 2); settings_graph_default.SamplesYAuto = SW_BOOL_FALSE; }
+        else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting Samples_Y."      , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
+      else if (strcmp(setkey, "SAMPLES_Y_AUTO")==0)
+        if  ((i=FetchSettingByName(setvalue,SW_BOOL_INT,SW_BOOL_STR))>0)                         settings_graph_default.SamplesYAuto  = i;
+        else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting Samples_Y_Auto." , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
       else if (strcmp(setkey, "TERMANTIALIAS")==0)
         if ((i=FetchSettingByName(setvalue,SW_ONOFF_INT, SW_ONOFF_STR ))>0)                      settings_term_default .TermAntiAlias = i;
         else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting TermAntiAlias.", linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
@@ -459,8 +514,9 @@ void ReadConfigFile(char *ConfigFname)
     else if (state == 7) // [units] section
      {
 
-#define GET_UNITNAME(output, last, type, sep) \
-      if (isalpha(setkey[i])) do { i++; } while ((isalnum(setkey[i])) || (setkey[i]=='_')); \
+#define GET_UNITNAME(output, last, type, sep, LaTeX) \
+      if (!LaTeX) { if (isalpha(setkey[i])) do { i++; } while ((isalnum(setkey[i])) || (setkey[i]=='_')); } \
+      else        { while ((setkey[i]!=sep)&&(setkey[i]!='\0')) i++; } \
       if (i==j) \
        { \
         if (&last==&output) \
@@ -484,13 +540,13 @@ void ReadConfigFile(char *ConfigFname)
       i=j=0;
       if (ppl_unit_pos == UNITS_MAX) { sprintf(temp_err_string, "Error in line %d of configuration file %s: Unit definition list full.", linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
 
-      GET_UNITNAME( ppl_unit_database[ppl_unit_pos].nameFs  , ppl_unit_database[ppl_unit_pos].nameFs  , "unit"    , '/' );
-      GET_UNITNAME( ppl_unit_database[ppl_unit_pos].nameAs  , ppl_unit_database[ppl_unit_pos].nameFs  , "unit"    , '/' );
-      GET_UNITNAME( ppl_unit_database[ppl_unit_pos].nameLs  , ppl_unit_database[ppl_unit_pos].nameAs  , "unit"    , '/' );
-      GET_UNITNAME( ppl_unit_database[ppl_unit_pos].nameFp  , ppl_unit_database[ppl_unit_pos].nameFs  , "unit"    , '/' );
-      GET_UNITNAME( ppl_unit_database[ppl_unit_pos].nameAp  , ppl_unit_database[ppl_unit_pos].nameAs  , "unit"    , ':' );
-      GET_UNITNAME( ppl_unit_database[ppl_unit_pos].nameLp  , ppl_unit_database[ppl_unit_pos].nameAp  , "unit"    , ':' );
-      GET_UNITNAME( ppl_unit_database[ppl_unit_pos].quantity, ppl_unit_database[ppl_unit_pos].quantity, "quantity", ' ' );
+      GET_UNITNAME( ppl_unit_database[ppl_unit_pos].nameFs  , ppl_unit_database[ppl_unit_pos].nameFs  , "unit"    , '/' , 0);
+      GET_UNITNAME( ppl_unit_database[ppl_unit_pos].nameAs  , ppl_unit_database[ppl_unit_pos].nameFs  , "unit"    , '/' , 0);
+      GET_UNITNAME( ppl_unit_database[ppl_unit_pos].nameLs  , ppl_unit_database[ppl_unit_pos].nameAs  , "unit"    , '/' , 1);
+      GET_UNITNAME( ppl_unit_database[ppl_unit_pos].nameFp  , ppl_unit_database[ppl_unit_pos].nameFs  , "unit"    , '/' , 0);
+      GET_UNITNAME( ppl_unit_database[ppl_unit_pos].nameAp  , ppl_unit_database[ppl_unit_pos].nameAs  , "unit"    , '/' , 0);
+      GET_UNITNAME( ppl_unit_database[ppl_unit_pos].nameLp  , ppl_unit_database[ppl_unit_pos].nameAp  , "unit"    , ':' , 1);
+      GET_UNITNAME( ppl_unit_database[ppl_unit_pos].quantity, ppl_unit_database[ppl_unit_pos].quantity, "quantity", ' ' , 0);
 
       if (setvalue[0]=='\0')
        {
@@ -538,7 +594,7 @@ void ReadConfigFile(char *ConfigFname)
       label_list_destroy(&label_list);
       label_list_copy(&label_list, &label_list_default);
       ppl_error_setstreaminfo(linecounter, "configuration file");
-      scriptcmd = parse(linebuffer);
+      scriptcmd = parse(linebuffer, 0);
       if (scriptcmd != NULL)
        {
         DictLookup(scriptcmd,"directive",NULL,(void **)(&StringScan));
@@ -602,6 +658,12 @@ void ReadConfigFile(char *ConfigFname)
       else if (strcmp(setkey, "AXES_MINTICKLEN"     )==0)
         if  (fl=GetFloat(setvalue, &i), ((gsl_finite(fl))&&(i==strlen(setvalue)))) EPS_AXES_MINTICKLEN   = fl * EPS_BASE_AXES_MINTICKLEN;
         else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting Axes_MinTickLen."      , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
+      else if (strcmp(setkey, "COLOURSCALE_MARGIN"  )==0)
+        if  (fl=GetFloat(setvalue, &i), ((gsl_finite(fl))&&(i==strlen(setvalue)))) EPS_COLOURSCALE_MARGIN= fl * EPS_BASE_COLOURSCALE_MARG;
+        else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting ColourScale_Margin."   , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
+      else if (strcmp(setkey, "COLOURSCALE_WIDTH"   )==0)
+        if  (fl=GetFloat(setvalue, &i), ((gsl_finite(fl))&&(i==strlen(setvalue)))) EPS_COLOURSCALE_WIDTH = fl * EPS_BASE_COLOURSCALE_WIDTH;
+        else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting ColourScale_Width."    , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
       else if (strcmp(setkey, "GRID_MAJLINEWIDTH"      )==0)
         if  (fl=GetFloat(setvalue, &i), ((gsl_finite(fl))&&(i==strlen(setvalue)))) EPS_GRID_MAJLINEWIDTH    = fl * EPS_BASE_GRID_MAJLINEWIDTH;
         else {sprintf(temp_err_string, "Error in line %d of configuration file %s: Illegal value for setting Grid_MajLineWidth."    , linecounter, ConfigFname); ppl_warning(ERR_PREFORMED, temp_err_string); continue; }
