@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: ppl_settings.c 811 2010-05-25 15:17:48Z dcf21 $
+// $Id: ppl_settings.c 894 2010-07-16 22:02:28Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -146,7 +146,36 @@ void ppl_settings_makedefault()
   settings_graph_default.BoxFromAuto           = 1;
   ppl_units_zero(&(settings_graph_default.BoxWidth));
   settings_graph_default.BoxWidthAuto          = 1;
+  strcpy(settings_graph_default.c1format, "");
+  settings_graph_default.c1formatset           = 0;
+  strcpy(settings_graph_default.c1label, "");
+  settings_graph_default.c1LabelRotate         = 0.0;
+  settings_graph_default.c1TickLabelRotate     = 0.0;
+  settings_graph_default.c1TickLabelRotation   = SW_TICLABDIR_HORI;
   settings_graph_default.clip                  = SW_ONOFF_OFF;
+  for (i=0; i<4; i++)
+   {
+    settings_graph_default.Clog[i]             = SW_BOOL_FALSE;
+    ppl_units_zero(&settings_graph_default.Cmax[i]);
+    settings_graph_default.Cmaxauto[i]         = SW_BOOL_TRUE;
+    ppl_units_zero(&settings_graph_default.Cmin[i]);
+    settings_graph_default.Cminauto[i]         = SW_BOOL_TRUE;
+    settings_graph_default.Crenorm[i]          = SW_BOOL_TRUE;
+    settings_graph_default.Creverse[i]         = SW_BOOL_FALSE;
+   }
+  settings_graph_default.ColKey                = SW_ONOFF_ON;
+  settings_graph_default.ColKeyPos             = SW_COLKEYPOS_R;
+  settings_graph_default.ColMapColSpace        = SW_COLSPACE_RGB;
+  strcpy(settings_graph_default.ColMapExpr1, "(c1)");
+  strcpy(settings_graph_default.ColMapExpr2, "(c1)");
+  strcpy(settings_graph_default.ColMapExpr3, "(c1)");
+  strcpy(settings_graph_default.ColMapExpr4, "");
+  strcpy(settings_graph_default.MaskExpr   , "");
+  settings_graph_default.ContoursLabel         = SW_ONOFF_OFF;
+  settings_graph_default.ContoursListLen       = -1;
+  for (i=0; i<MAX_CONTOURS; i++) settings_graph_default.ContoursList[i] = 0.0;
+  settings_graph_default.ContoursN             = 12;
+  ppl_units_zero(&settings_graph_default.ContoursUnit);
   with_words_zero(&(settings_graph_default.DataStyle),1);
   settings_graph_default.DataStyle.linespoints = SW_STYLE_POINTS;
   settings_graph_default.FontSize              = 1.0;
@@ -189,6 +218,11 @@ void ppl_settings_makedefault()
   settings_graph_default.PointLineWidth= 1.0;
   settings_graph_default.projection    = SW_PROJ_FLAT;
   settings_graph_default.samples       = 250;
+  settings_graph_default.SamplesX      = 40;
+  settings_graph_default.SamplesXAuto  = SW_BOOL_FALSE;
+  settings_graph_default.SamplesY      = 40;
+  settings_graph_default.SamplesYAuto  = SW_BOOL_FALSE;
+  settings_graph_default.Sample2DMethod= SW_SAMPLEMETHOD_NEAREST;
   settings_graph_default.TextColour    = COLOUR_BLACK;
   settings_graph_default.TextColour1   = 0;
   settings_graph_default.TextColour2   = 0;
@@ -447,10 +481,10 @@ void ppl_settings_readconfig()
 
 int colour_fromdict    (Dict *in, char *prefix, int *outcol, int *outcolspace,
                         double *outcol1, double *outcol2, double *outcol3, double *outcol4,
-                        char **outcol1S, char **outcol2S, char **outcol3S, char **outcol4S,
+                        char **outcolS, char **outcol1S, char **outcol2S, char **outcol3S, char **outcol4S,
                         unsigned char *USEcol, unsigned char *USEcol1234, int *errpos, unsigned char malloced)
  {
-  char *tempstr, DictName[32];
+  char *tempstr, *tempstre, DictName[32];
   int   cindex, i, j, palette_index;
   void *tmp;
   value valobj;
@@ -477,6 +511,12 @@ int colour_fromdict    (Dict *in, char *prefix, int *outcol, int *outcolspace,
   sprintf(DictName, "%scolourYexpr", prefix);  DictLookup(in,DictName,NULL,(void **)&tempstrY);
   sprintf(DictName, "%scolourKexpr", prefix);  DictLookup(in,DictName,NULL,(void **)&tempstrK);
 
+  // Decide whether colour expression contains $columns and needs postponing
+  tempstre=NULL;
+  if ((tempstr!=NULL) && (outcolS!=NULL))
+   for (j=0; tempstr[j]!='\0'; j++)
+    if (tempstr[j]=='$') { tempstre=tempstr; tempstr=NULL; break; }
+
   if (tempstr != NULL) // Colour is specified by name or by palette index
    {
     StrStrip(tempstr,tempstr);
@@ -505,17 +545,42 @@ int colour_fromdict    (Dict *in, char *prefix, int *outcol, int *outcolspace,
       *outcol4     = settings_palette4_current[palette_index];
      }
     *outcol  = cindex;
+    if (malloced && (outcolS !=NULL) && (*outcolS !=NULL)) free(*outcolS );
     if (malloced && (outcol1S!=NULL) && (*outcol1S!=NULL)) free(*outcol1S);
     if (malloced && (outcol2S!=NULL) && (*outcol2S!=NULL)) free(*outcol2S);
     if (malloced && (outcol3S!=NULL) && (*outcol3S!=NULL)) free(*outcol3S);
     if (malloced && (outcol4S!=NULL) && (*outcol4S!=NULL)) free(*outcol4S);
+    if (outcolS !=NULL) *outcolS =NULL;
     if (outcol1S!=NULL) *outcol1S=NULL;
     if (outcol2S!=NULL) *outcol2S=NULL;
     if (outcol3S!=NULL) *outcol3S=NULL;
     if (outcol4S!=NULL) *outcol4S=NULL;
     if (USEcol    !=NULL) *USEcol     = (cindex> 0);
     if (USEcol1234!=NULL) *USEcol1234 = (cindex==0);
-   } else if ((tempvalR!=NULL) || (tempvalH!=NULL) || (tempvalC!=NULL)) { // Colour is specified by RGB/HSB/CMYK components
+   }
+  else if (tempstre!=NULL)
+   {
+    if (USEcol    !=NULL) *USEcol     = 0;
+    if (USEcol1234!=NULL) *USEcol1234 = 0;
+    if (outcolS   ==NULL) { ppl_error(ERR_INTERNAL, -1, -1, "Received colour expressions, but have not received strings to put them into."); return 1; }
+    if (malloced)
+     {
+      if (*outcolS !=NULL) free(*outcolS );
+      if (*outcol1S!=NULL) free(*outcol1S);
+      if (*outcol2S!=NULL) free(*outcol2S);
+      if (*outcol3S!=NULL) free(*outcol3S);
+      if (*outcol4S!=NULL) free(*outcol4S);
+      *outcol1S = *outcol2S = *outcol3S = *outcol4S = NULL;
+      *outcolS  = (char *)COLMALLOC(strlen(tempstre)+1); strcpy(*outcolS , tempstre);
+     }
+    else
+     {
+      *outcol1S = *outcol2S = *outcol3S = *outcol4S = NULL;
+      *outcolS  = tempstre;
+     }
+   }
+  else if ((tempvalR!=NULL) || (tempvalH!=NULL) || (tempvalC!=NULL)) // Colour is specified by RGB/HSB/CMYK components
+   {
 
 #define CHECK_REAL_DIMLESS(X) \
    { \
@@ -552,10 +617,12 @@ int colour_fromdict    (Dict *in, char *prefix, int *outcol, int *outcolspace,
      }
     if (USEcol    !=NULL) *USEcol     = 0;
     if (USEcol1234!=NULL) *USEcol1234 = 1;
+    if (malloced && (outcolS !=NULL) && (*outcolS !=NULL)) free(*outcolS );
     if (malloced && (outcol1S!=NULL) && (*outcol1S!=NULL)) free(*outcol1S);
     if (malloced && (outcol2S!=NULL) && (*outcol2S!=NULL)) free(*outcol2S);
     if (malloced && (outcol3S!=NULL) && (*outcol3S!=NULL)) free(*outcol3S);
     if (malloced && (outcol4S!=NULL) && (*outcol4S!=NULL)) free(*outcol4S);
+    if (outcolS !=NULL) *outcolS =NULL;
     if (outcol1S!=NULL) *outcol1S=NULL;
     if (outcol2S!=NULL) *outcol2S=NULL;
     if (outcol3S!=NULL) *outcol3S=NULL;
@@ -566,11 +633,12 @@ int colour_fromdict    (Dict *in, char *prefix, int *outcol, int *outcolspace,
     if (outcol1S  ==NULL) { ppl_error(ERR_INTERNAL, -1, -1, "Received colour expressions, but have not received strings to put them into."); return 1; }
     if (malloced)
      {
+      if (*outcolS !=NULL) free(*outcolS );
       if (*outcol1S!=NULL) free(*outcol1S);
       if (*outcol2S!=NULL) free(*outcol2S);
       if (*outcol3S!=NULL) free(*outcol3S);
       if (*outcol4S!=NULL) free(*outcol4S);
-      *outcol4S = NULL;
+      *outcolS = *outcol4S = NULL;
       if (tempstrR!=NULL)
        {
         *outcolspace = SW_COLSPACE_RGB;
@@ -596,7 +664,7 @@ int colour_fromdict    (Dict *in, char *prefix, int *outcol, int *outcolspace,
      }
     else
      {
-      *outcol4S = NULL;
+      *outcolS = *outcol4S = NULL;
       if (tempstrR!=NULL)
        {
         *outcolspace = SW_COLSPACE_RGB;
@@ -952,10 +1020,10 @@ void label_add(label_object **list, Dict *in)
   // Check for halign or valign modifiers
   DictLookup(in,"halign",NULL,(void **)&tempstr);
   if (tempstr != NULL) out->HAlign = FetchSettingByName(tempstr, SW_HALIGN_INT, SW_HALIGN_STR);
-  else                 out->HAlign = 0;
+  else                 out->HAlign = settings_graph_current.TextHAlign;
   DictLookup(in,"valign",NULL,(void **)&tempstr);
   if (tempstr != NULL) out->VAlign = FetchSettingByName(tempstr, SW_VALIGN_INT, SW_VALIGN_STR);
-  else                 out->VAlign = 0;
+  else                 out->VAlign = settings_graph_current.TextVAlign;
 
   if (ang != NULL) out->rotation = ang->real;
   else             out->rotation = 0.0;
@@ -1129,8 +1197,9 @@ void with_words_zero(with_words *a, const unsigned char malloced)
   a->linewidth = a->pointlinewidth = a->pointsize = 1.0;
   a->colour1 = a->colour2 = a->colour3 = a->colour4 = a->fillcolour1 = a->fillcolour2 = a->fillcolour3 = a->fillcolour4 = 0.0;
   a->STRlinetype = a->STRlinewidth = a->STRpointlinewidth = a->STRpointsize = a->STRpointtype = NULL;
-  a->STRcolour1 = a->STRcolour2 = a->STRcolour3 = a->STRcolour4 = a->STRfillcolour1 = a->STRfillcolour2 = a->STRfillcolour3 = a->STRfillcolour4 = NULL;
+  a->STRcolour = a->STRcolour1 = a->STRcolour2 = a->STRcolour3 = a->STRcolour4 = a->STRfillcolour = a->STRfillcolour1 = a->STRfillcolour2 = a->STRfillcolour3 = a->STRfillcolour4 = NULL;
   a->USEcolour = a->USEfillcolour = a->USElinespoints = a->USElinetype = a->USElinewidth = a->USEpointlinewidth = a->USEpointsize = a->USEpointtype = a->USEstyle = a->USEcolour1234 = a->USEfillcolour1234 = 0;
+  a->AUTOcolour = a->AUTOlinetype = a->AUTOpointtype = 0;
   a->malloced = malloced;
   return;
  }
@@ -1147,10 +1216,10 @@ void with_words_fromdict(Dict *in, with_words *out, const unsigned char MallocNe
 
   // read colour names
   colour_fromdict(in,""    ,&out->colour    ,&out->Col1234Space    ,&out->colour1        ,&out->colour2        ,&out->colour3        ,&out->colour4       ,
-                                                                    &out->STRcolour1     ,&out->STRcolour2     ,&out->STRcolour3     ,&out->STRcolour4    ,
+                                               &out->STRcolour     ,&out->STRcolour1     ,&out->STRcolour2     ,&out->STRcolour3     ,&out->STRcolour4    ,
                   &out->USEcolour    ,&out->USEcolour1234    ,&i,MallocNew);
   colour_fromdict(in,"fill",&out->fillcolour,&out->FillCol1234Space,&out->fillcolour1    ,&out->fillcolour2    ,&out->fillcolour3    ,&out->fillcolour4   ,
-                                                                    &out->STRfillcolour1 ,&out->STRfillcolour2 ,&out->STRfillcolour3 ,&out->STRfillcolour4,
+                                            &out->STRfillcolour    ,&out->STRfillcolour1 ,&out->STRfillcolour2 ,&out->STRfillcolour3 ,&out->STRfillcolour4,
                   &out->USEfillcolour,&out->USEfillcolour1234,&i,MallocNew);
 
   // Other settings
@@ -1312,23 +1381,24 @@ void with_words_merge(with_words *out, const with_words *a, const with_words *b,
         out->style = x->style; out->USEstyle = 1;
        }
      }
-    if (x->STRcolour1       !=NULL) { out->STRcolour1 = x->STRcolour1; out->STRcolour2 = x->STRcolour2; out->STRcolour3 = x->STRcolour3; out->STRcolour4 = x->STRcolour4; out->Col1234Space = x->Col1234Space; out->USEcolour1234 = 0; out->USEcolour = 0; }
-    if (x->USEcolour1234          ) { out->colour1 = x->colour1; out->colour2 = x->colour2; out->colour3 = x->colour3; out->colour4 = x->colour4; out->Col1234Space = x->Col1234Space; out->USEcolour1234 = 1; out->USEcolour = 0; out->STRcolour1 = out->STRcolour2 = out->STRcolour3 = out->STRcolour4 = NULL; }
-    if (x->USEcolour              ) { out->colour = x->colour; out->USEcolour = 1; out->USEcolour1234 = 0; out->STRcolour1 = out->STRcolour2 = out->STRcolour3 = out->STRcolour4 = NULL; }
+    if (x->USEcolour1234          ) { out->colour1 = x->colour1; out->colour2 = x->colour2; out->colour3 = x->colour3; out->colour4 = x->colour4; out->Col1234Space = x->Col1234Space; out->USEcolour1234 = 1; out->USEcolour = 0; out->STRcolour1 = out->STRcolour2 = out->STRcolour3 = out->STRcolour4 = NULL; out->AUTOcolour = x->AUTOcolour; }
+    if (x->USEcolour              ) { out->colour = x->colour; out->USEcolour = 1; out->USEcolour1234 = 0; out->STRcolour1 = out->STRcolour2 = out->STRcolour3 = out->STRcolour4 = NULL; out->AUTOcolour = x->AUTOcolour; }
+    if (x->STRcolour        !=NULL) { out->STRcolour = x->STRcolour; }
+    if (x->STRcolour1       !=NULL) { out->STRcolour1 = x->STRcolour1; out->STRcolour2 = x->STRcolour2; out->STRcolour3 = x->STRcolour3; out->STRcolour4 = x->STRcolour4; out->Col1234Space = x->Col1234Space; }
     if (x->STRfillcolour1   !=NULL) { out->STRfillcolour1 = x->STRfillcolour1; out->STRfillcolour2 = x->STRfillcolour2; out->STRfillcolour3 = x->STRfillcolour3; out->STRfillcolour4 = x->STRfillcolour4; out->FillCol1234Space = x->FillCol1234Space; out->USEfillcolour1234 = 0; out->USEfillcolour = 0; }
     if (x->USEfillcolour1234      ) { out->fillcolour1 = x->fillcolour1; out->fillcolour2 = x->fillcolour2; out->fillcolour3 = x->fillcolour3; out->fillcolour4 = x->fillcolour4; out->FillCol1234Space = x->FillCol1234Space; out->USEfillcolour1234 = 1; out->USEfillcolour = 0; out->STRfillcolour1 = out->STRfillcolour2 = out->STRfillcolour3 = out->STRfillcolour4 = NULL; }
     if (x->USEfillcolour          ) { out->fillcolour = x->fillcolour; out->USEfillcolour = 1; out->USEfillcolour1234 = 0; out->STRfillcolour1 = out->STRfillcolour2 = out->STRfillcolour3 = out->STRfillcolour4 = NULL; }
     if (x->USElinespoints         ) { out->linespoints = x->linespoints; out->USElinespoints = 1; }
-    if (x->STRlinetype      !=NULL) { out->STRlinetype = x->STRlinetype; out->USElinetype = 0; }
-    if (x->USElinetype            ) { out->linetype = x->linetype; out->USElinetype = 1; out->STRlinetype = NULL; }
+    if (x->USElinetype            ) { out->linetype = x->linetype; out->USElinetype = 1; out->STRlinetype = NULL; out->AUTOlinetype = x->AUTOlinetype; }
+    if (x->STRlinetype      !=NULL) { out->STRlinetype = x->STRlinetype; }
     if (x->STRlinewidth     !=NULL) { out->STRlinewidth = x->STRlinewidth; out->USElinewidth = 0; }
     if (x->USElinewidth           ) { out->linewidth = x->linewidth; out->USElinewidth = 1; out->STRlinewidth = NULL; }
     if (x->STRpointlinewidth!=NULL) { out->STRpointlinewidth = x->STRpointlinewidth; out->USEpointlinewidth = 0; }
     if (x->USEpointlinewidth      ) { out->pointlinewidth = x->pointlinewidth; out->USEpointlinewidth = 1; out->STRpointlinewidth = NULL; }
-    if (x->STRpointsize     !=NULL) { out->STRpointsize = x->STRpointsize; out->USEpointsize = 0; }
+    if (x->STRpointsize     !=NULL) { out->STRpointsize = x->STRpointsize; }
     if (x->USEpointsize           ) { out->pointsize = x->pointsize; out->USEpointsize = 1; out->STRpointsize = NULL; }
-    if (x->STRpointtype     !=NULL) { out->STRpointtype = x->STRpointtype; out->USEpointtype = 0; }
-    if (x->USEpointtype           ) { out->pointtype = x->pointtype; out->USEpointtype = 1; out->STRpointtype = NULL; }
+    if (x->USEpointtype           ) { out->pointtype = x->pointtype; out->USEpointtype = 1; out->STRpointtype = NULL; out->AUTOpointtype = x->AUTOpointtype; }
+    if (x->STRpointtype     !=NULL) { out->STRpointtype = x->STRpointtype; }
    }
   return;
  }
@@ -1470,7 +1540,7 @@ void CopyAxis(settings_axis *out, const settings_axis *in)
 
 void CopyAxisTics(settings_axis *out, const settings_axis *in)
  {
-  int   i,j;
+  int   i=0,j;
   void *tmp;
   if (in->TickStrs != NULL)
    {
@@ -1489,7 +1559,7 @@ void CopyAxisTics(settings_axis *out, const settings_axis *in)
 
 void CopyAxisMTics(settings_axis *out, const settings_axis *in)
  {
-  int   i,j;
+  int   i=0,j;
   void *tmp;
   if (in->MTickStrs != NULL)
    {
