@@ -5,8 +5,9 @@
 //
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
+//               2010      Zoltan Voros
 //
-// $Id: ppl_shell.c 761 2010-05-12 22:03:11Z dcf21 $
+// $Id: ppl_shell.c 948 2010-09-04 17:25:15Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -25,6 +26,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <signal.h>
 #include <errno.h>
 #include <wordexp.h>
 #include <glob.h>
@@ -79,43 +81,44 @@ void InteractiveSession()
   if (DEBUG) ppl_log("Starting an interactive session.");
   PPL_FLOWCTRL_LOOPNAME[0] = NULL;
 
-  // Set up SIGINT handler
-  if (sigsetjmp(sigjmp_ToInteractive, 1) == 0)
+  if ((isatty(STDIN_FILENO) == 1) && (settings_session_default.splash == SW_ONOFF_ON)) ppl_report(txt_init);
+
+  PPL_SHELL_EXITING = 0;
+  ClearInputSource(NULL,NULL,NULL,&OldLB,&OldLBP,&OldLBA);
+  while ((PPL_SHELL_EXITING == 0) && (PPL_FLOWCTRL_BROKEN == 0) && (PPL_FLOWCTRL_CONTINUED == 0))
    {
-    sigjmp_FromSigInt = &sigjmp_ToInteractive;
-
-    if ((isatty(STDIN_FILENO) == 1) && (settings_session_default.splash == SW_ONOFF_ON)) ppl_report(txt_init);
-
-    PPL_SHELL_EXITING = 0;
-    ClearInputSource(NULL,NULL,NULL,&OldLB,&OldLBP,&OldLBA);
-    while ((PPL_SHELL_EXITING == 0) && (PPL_FLOWCTRL_BROKEN == 0) && (PPL_FLOWCTRL_CONTINUED == 0))
+    // Set up SIGINT handler
+    if (sigsetjmp(sigjmp_ToInteractive, 1) == 0)
      {
+      sigjmp_FromSigInt = &sigjmp_ToInteractive;
+
       CheckForGvOutput();
       if (isatty(STDIN_FILENO) == 1) SetInputSourceReadline(&linenumber);
       else                           SetInputSourcePipe(&linenumber, "piped input");
       line_ptr = FetchInputStatement("pyxplot> ",".......> ");
       if (line_ptr == NULL) break;
+      NestedConditionalDepth=0; // Reset nested conditional depth after SIGINT
       ProcessDirective(line_ptr, isatty(STDIN_FILENO), 0);
       ppl_error_setstreaminfo(-1, "");
       PPLKillAllHelpers();
+     } else {
+      sigemptyset(&sigs); // SIGINT longjmps return here
+      sigaddset(&sigs,SIGCHLD);
+      sigprocmask(SIG_UNBLOCK, &sigs, NULL);
+      fprintf(stdout,"\n");
+      if (chdir(settings_session_default.cwd) < 0) { ppl_fatal(__FILE__,__LINE__,"chdir into cwd failed."); } // chdir out of temporary directory
      }
-
-    if (isatty(STDIN_FILENO) == 1)
-     {
-      if (settings_session_default.splash == SW_ONOFF_ON) ppl_report("\nGoodbye. Have a nice day.");
-      else                                                ppl_report(""); // Make a new line
-     }
-   } else {
-    sigemptyset(&sigs);
-    sigaddset(&sigs,SIGCHLD);
-    sigprocmask(SIG_UNBLOCK, &sigs, NULL);
-    ppl_error(ERR_PREFORMED, -1, -1, "\nReceived CTRL-C. Terminating session."); // SIGINT longjmps return here
-    if (chdir(settings_session_default.cwd) < 0) { ppl_fatal(__FILE__,__LINE__,"chdir into cwd failed."); } // chdir out of temporary directory
    }
-  PPL_SHELL_EXITING = 0;
-  ClearInputSource(OldLB,OldLBP,OldLBA,NULL,NULL,NULL);
 
   sigjmp_FromSigInt = &sigjmp_ToMain; // SIGINT now drops back through to main().
+  PPL_SHELL_EXITING = 0;
+  ClearInputSource(OldLB,OldLBP,OldLBA,NULL,NULL,NULL);
+  if (isatty(STDIN_FILENO) == 1)
+   {
+    if (settings_session_default.splash == SW_ONOFF_ON) ppl_report("\nGoodbye. Have a nice day.");
+    else                                                ppl_report(""); // Make a new line
+   }
+
   return;
  }
 
@@ -171,10 +174,12 @@ void ProcessPyXPlotScript(char *input, int IterLevel)
 
 int ProcessDirective(char *in, int interactive, int IterLevel)
  {
-  int   memcontext, i, is, j, l, breakable;
+  int   memcontext, i, is, j, l, breakable, k;
   int   status=0;
   char  QuoteChar='\0';
-  char *DirectiveLinebuffer = NULL;
+  char *DirectiveLinebuffer = NULL, *DirectiveLinebuffer2 = NULL;
+  char *expmac;
+  value *VarData = NULL;
   Dict *command;
   FILE *SubstPipe;
   sigset_t sigs;
@@ -196,8 +201,9 @@ int ProcessDirective(char *in, int interactive, int IterLevel)
     // Do `` substitution
     l = strlen(in+1);
     DirectiveLinebuffer = (char *)lt_malloc(l*sizeof(char));
+
     if (DirectiveLinebuffer==NULL) ppl_fatal(__FILE__,__LINE__,"Out of memory.");
-    for (i=0, j=0; in[i]!='\0'; i++)
+    for (i=0, j=0, QuoteChar='\0'; in[i]!='\0'; i++)
      {
       if      ((QuoteChar=='\0') && (in[i]=='\'')                   ) QuoteChar = '\'';
       else if ((QuoteChar=='\0') && (in[i]=='\"')                   ) QuoteChar = '\"';
@@ -259,11 +265,46 @@ int ProcessDirective(char *in, int interactive, int IterLevel)
      }
     DirectiveLinebuffer[j] = '\0';
 
+    // Substitute for macros
+    l                    = strlen(DirectiveLinebuffer);
+    expmac               = (char *)lt_malloc(l*sizeof(char));
+    DirectiveLinebuffer2 = (char *)lt_malloc(LSTR_LENGTH*sizeof(char));
+
+    for (i=0, k=0, QuoteChar='\0'; DirectiveLinebuffer[i]!='\0'; )
+     {
+      if      ((QuoteChar=='\0') && (DirectiveLinebuffer[i]=='\'')                       ) QuoteChar = '\'';
+      else if ((QuoteChar=='\0') && (DirectiveLinebuffer[i]=='\"')                       ) QuoteChar = '\"';
+      else if ((QuoteChar=='\'') && (DirectiveLinebuffer[i]=='\'') && (DirectiveLinebuffer[i-1]!='\\')) QuoteChar = '\0';
+      else if ((QuoteChar=='\"') && (DirectiveLinebuffer[i]=='\"') && (DirectiveLinebuffer[i-1]!='\\')) QuoteChar = '\0';
+
+      if ((DirectiveLinebuffer[i]!='@')||(QuoteChar!='\0'))
+       {
+        if (k>=LSTR_LENGTH-1) { ppl_error(ERR_SYNTAX, -1, -1, "Line length overflow."); break; }
+        DirectiveLinebuffer2[k++] = DirectiveLinebuffer[i++];
+       }
+      else
+       {
+        for (j=0, l=++i; (isalnum(DirectiveLinebuffer[l])) || (DirectiveLinebuffer[l]=='_'); ) expmac[j++] = DirectiveLinebuffer[l++];
+        expmac[j] = '\0';
+        DictLookup(_ppl_UserSpace_Vars, expmac, NULL, (void **)&VarData);
+        if (VarData == NULL) { sprintf(temp_err_string, "Undefined macro, \"%s\".", expmac); ppl_warning(ERR_SYNTAX, temp_err_string); DirectiveLinebuffer2[k++] = '@'; continue; }
+        if (VarData->string == NULL) { sprintf(temp_err_string, "Attempt to expand a macro, \"%s\", which is a numerical variable not a string.", expmac); ppl_warning(ERR_SYNTAX, temp_err_string); DirectiveLinebuffer2[k++] = '@'; continue; }
+        j=strlen(VarData->string);
+        i=l;
+        for (l=0; ((l<j)&&(k<LSTR_LENGTH-1)); l++)
+         {
+          DirectiveLinebuffer2[k++] = VarData->string[l];
+         }
+        if (k>=LSTR_LENGTH-1) { ppl_error(ERR_SYNTAX, -1, -1, "Line length overflow."); break; }
+       }
+     }
+    DirectiveLinebuffer2[k] = '\0';
+
     // Parse and execute command
     if (status==0)
      {
-      command = parse(DirectiveLinebuffer);
-      if (command != NULL) status = ProcessDirective2(DirectiveLinebuffer, command, interactive, memcontext, IterLevel);
+      command = parse(DirectiveLinebuffer2, IterLevel);
+      if (command != NULL) status = ProcessDirective2(DirectiveLinebuffer2, command, interactive, memcontext, IterLevel);
       else                 status = 1;
       // If command is NULL, we had a syntax error
      }
@@ -303,7 +344,7 @@ int ProcessDirective2(char *in, Dict *command, int interactive, int memcontext,
   if      (strcmp(directive, "pling")==0)
    {
     DictLookup(command,"cmd",NULL,(void **)(&directive));
-    system(directive);
+    if (system(directive)) { if (DEBUG) ppl_log("Pling command received non-zero return value."); }
    }
   else if (strcmp(directive, "var_set")==0)
    {
@@ -326,6 +367,8 @@ int ProcessDirective2(char *in, Dict *command, int interactive, int memcontext,
    return directive_interpolate(command,INTERP_AKIMA);
   else if ( (strcmp(directive, "arrow")==0) || (strcmp(directive, "line")==0) )
    directive_arrow(command, interactive);
+  else if (strcmp(directive, "assert")==0)
+   return directive_assert(command);
   else if (strcmp(directive, "box")==0)
    directive_box(command, interactive);
   else if (strcmp(directive, "break")==0)
@@ -370,6 +413,19 @@ int ProcessDirective2(char *in, Dict *command, int interactive, int memcontext,
    return directive_if(command, IterLevel+1);
   else if (strcmp(directive, "image")==0)
    directive_image(command, interactive);
+  else if (strcmp(directive, "interpolate2d")==0)
+   {
+    int type = INTERP_2D;
+    char *tempstr;
+    DictLookup(command,"bmp",NULL,(void **)(&tempstr));
+    if (tempstr!=NULL) switch (tempstr[4])
+     {
+      case 'r': type = INTERP_BMPR; break;
+      case 'g': type = INTERP_BMPG; break;
+      case 'b': type = INTERP_BMPB; break;
+     }
+    return directive_interpolate(command,type);
+   }
   else if (strcmp(directive, "linear")==0)
    return directive_interpolate(command,INTERP_LINEAR);
   else if (strcmp(directive, "list")==0)
@@ -460,6 +516,8 @@ int ProcessDirective2(char *in, Dict *command, int interactive, int memcontext,
    directive_solve(command);
   else if (strcmp(directive, "spline")==0)
    return directive_interpolate(command,INTERP_SPLINE);
+  else if (strcmp(directive, "stepwise")==0)
+   return directive_interpolate(command,INTERP_STEPWISE);
   else if (strcmp(directive, "subroutine")==0)
    return directive_subroutine(command, IterLevel);
   else if (strcmp(directive, "swap")==0)
@@ -492,6 +550,63 @@ int ProcessDirective2(char *in, Dict *command, int interactive, int memcontext,
   return 0;
  }
 
+int directive_assert(Dict *command)
+ {
+  char  *txt, *version, *gtreq, *lt;
+  value *val;
+  DictLookup(command,"message",NULL,(void **)(&txt));
+  DictLookup(command,"gtreq"  ,NULL,(void **)(&gtreq));
+  DictLookup(command,"lt"     ,NULL,(void **)(&lt));
+  DictLookup(command,"version",NULL,(void **)(&version));
+  DictLookup(command,"expr",NULL,(void **)(&val));
+  if (val != NULL)
+   {
+    if (txt==NULL) txt="Assertion was not true.";
+    if ((val->real==0.0)&&(val->imag==0.0))
+     {
+      sprintf(temp_err_string, "Assertion Error: %s", txt);
+      ppl_error(ERR_PREFORMED,-1,-1,temp_err_string);
+      return 1;
+     }
+   }
+  if (version != NULL)
+   {
+    int i=0,j=0,sgn,pass=0;
+    char txtauto[64];
+    sprintf(txtauto, "This script requires a%s version of PyXPlot (%s %s)", (lt==NULL)?" newer":"n older", (lt==NULL)?">=":"<", version);
+#define VERSION_FAIL { sprintf(temp_err_string, "Assertion Error: %s", txt); ppl_error(ERR_PREFORMED,-1,-1,temp_err_string); return 1; }
+    if (txt==NULL) txt=txtauto;
+    sgn = (lt==NULL)?1:-1;
+    while ((version[i]>'\0')&&(version[i]<=' ')) i++;
+    if (version[i]=='\0') return 0;
+    for (j=0; ((version[i]>='0')&&(version[i]<='9')); i++) { j=10*j+(version[i]-'0'); if (j>1e6)j=1e6; }
+    if ((sgn*VERSION_MAJ < sgn*j)&&(!pass)) VERSION_FAIL;
+    if  (sgn*VERSION_MAJ > sgn*j) pass=1;
+    while ((version[i]>'\0')&&(version[i]<=' ')) i++;
+    if      (version[i]=='.') i++;
+    else if (version[i]!='\0') { ppl_error(ERR_SYNTAX,-1,-1,"Malformed version string."); return 1; }
+    while ((version[i]>'\0')&&(version[i]<=' ')) i++;
+    if ((version[i]=='\0')&&(sgn==-1)&&(!pass)) VERSION_FAIL;
+    if (version[i]=='\0') return 0;
+    for (j=0; ((version[i]>='0')&&(version[i]<='9')); i++) { j=10*j+(version[i]-'0'); if (j>1e6)j=1e6; }
+    if ((sgn*VERSION_MIN < sgn*j)&&(!pass)) VERSION_FAIL;
+    if  (sgn*VERSION_MIN > sgn*j) pass=1;
+    while ((version[i]>'\0')&&(version[i]<=' ')) i++;
+    if      (version[i]=='.') i++;
+    else if (version[i]!='\0') { ppl_error(ERR_SYNTAX,-1,-1,"Malformed version string."); return 1; }
+    while ((version[i]>'\0')&&(version[i]<=' ')) i++;
+    if ((version[i]=='\0')&&(sgn==-1)&&(!pass)) VERSION_FAIL;
+    if (version[i]=='\0') return 0;
+    for (j=0; ((version[i]>='0')&&(version[i]<='9')); i++) { j=10*j+(version[i]-'0'); if (j>1e6)j=1e6; }
+    if ((sgn*VERSION_REV < sgn*j)&&(!pass)) VERSION_FAIL;
+    if  (sgn*VERSION_REV > sgn*j) pass=1;
+    while ((version[i]>'\0')&&(version[i]<=' ')) i++;
+    if (version[i]!='\0') { ppl_error(ERR_SYNTAX,-1,-1,"Malformed version string."); return 1; }
+    if ((version[i]=='\0')&&(sgn==-1)&&(!pass)) VERSION_FAIL;
+   }
+  return 0;
+ }
+
 int directive_break(Dict *command, int IterLevel)
  {
   char *loopname;
