@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: ppl_tabulate.c 740 2010-04-30 15:00:59Z dcf21 $
+// $Id: ppl_tabulate.c 866 2010-07-03 15:40:57Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -52,7 +52,7 @@
 static int DataGridDisplay(FILE *output, DataTable *data, int Ncolumns, value **min, value **max, char *format)
  {
   DataBlock *blk;
-  char *cptr, tmpchr;
+  char *cptr, tmpchr='\0';
   double tmpdbl, multiplier[USING_ITEMS_MAX];
   unsigned char InRange, split, AllInts[USING_ITEMS_MAX], AllSmall[USING_ITEMS_MAX];
   long int i,k;
@@ -302,7 +302,7 @@ int directive_tabulate(Dict *command, char *line)
         ContextDataTab = lt_DescendIntoNewContext();
 
         // Read data from file
-        DataFile_read(&data, &status, errtext, filename, *indexptr, rowcol, UsingList, EveryList, NULL, NUsingItems, SelectCrit, DATAFILE_DISCONTINUOUS, SortBy, DATAFILE_CONTINUOUS, &ErrCount);
+        DataFile_read(&data, &status, errtext, filename, *indexptr, rowcol, UsingList, 0, EveryList, NULL, NUsingItems, SelectCrit, DATAFILE_DISCONTINUOUS, SortBy, DATAFILE_CONTINUOUS, &ErrCount);
         if (status) { ppl_error(ERR_GENERAL, -1, -1, errtext); ppl_glob_close(glob_handle); fclose(output); return 1; }
         status = DataGridDisplay(output, data, NUsingItems, min, max, format);
         if (status) { ppl_glob_close(glob_handle); fclose(output); return 1; }
@@ -430,7 +430,8 @@ int directive_tabulate(Dict *command, char *line)
        }
       DataFile_FromFunctions(ordinate_raster, FlagParametric,
                              NumberOfSamples, (FlagParametric ? &para_Tunit : &raster_units),
-                             &data, &status, errtext, fnlist, k, UsingList, NULL, NUsingItems, SelectCrit, DATAFILE_DISCONTINUOUS, SortBy, DATAFILE_CONTINUOUS, &ErrCount);
+                             NULL, 0, NULL,
+                             &data, &status, errtext, fnlist, k, UsingList, 0, NULL, NUsingItems, SelectCrit, DATAFILE_DISCONTINUOUS, SortBy, DATAFILE_CONTINUOUS, &ErrCount);
       if (status) { ppl_error(ERR_GENERAL, -1, -1, errtext); fclose(output); return 1; }
       status = DataGridDisplay(output, data, NUsingItems, min+1, max+1, format); // First range is for ordinate axis
       if (status) { fclose(output); return 1; }
