@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: ppl_texify.c 734 2010-04-28 15:11:41Z dcf21 $
+// $Id: ppl_texify.c 948 2010-09-04 17:25:15Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -242,7 +242,7 @@ void texify_quotedstring(char *in, int *end, char *out, int EvalStrings, int *st
 
 void texify_algebra(char *in, int *end, char *out, int EvalStrings, int *status, char *errtext, int RecursionDepth, int *BracketLevel)
  {
-  int i, j, k, l, m, outpos=0, CalculatedEnd, OpenBracketPos, bufpos=0, BracketType, OldBracketLevel=0;
+  int i, j, k, l, m, outpos=0, CalculatedEnd, OpenBracketPos=0, bufpos=0, BracketType, OldBracketLevel=0;
   int prev_start, prev_end, next_start, next_end, prev_bufno, next_bufno;
   int PowLevel = 0;
   unsigned char ci,cj;
@@ -250,10 +250,10 @@ void texify_algebra(char *in, int *end, char *out, int EvalStrings, int *status,
   unsigned char OpList[OPLIST_LEN];                  // A list of what operations this expression contains
   unsigned char StatusRow       [ALGEBRA_MAXLENGTH]; // Describes the atoms at each position in the expression
   unsigned char StatusRowInitial[ALGEBRA_MAXLENGTH];
-  char          dummy[DUMMYVAR_MAXLEN], GreekifiedDummy[DUMMYVAR_MAXLEN+10], *LaTeXModel, DummyLaTeXModel[FNAME_LENGTH], *FunctionName;
+  char          dummy[DUMMYVAR_MAXLEN], GreekifiedDummy[DUMMYVAR_MAXLEN+10], *LaTeXModel, DummyLaTeXModel[FNAME_LENGTH], *FunctionName=NULL;
   int           CommaPositions[MAX_STR_FORMAT_ITEMS], Nargs, RequiredNargs;
-  int           LastItemStartPos;
-  unsigned char LastItemContainedSuperscript;
+  int           LastItemStartPos=0;
+  unsigned char LastItemContainedSuperscript=0;
   value ResultBuffer[ALGEBRA_MAXITEMS];       // A buffer of temporary numerical results
   DictIterator *DictIter;
 
@@ -583,11 +583,13 @@ void texify_algebra(char *in, int *end, char *out, int EvalStrings, int *status,
 
 void wrapper_texify(char *in, int inlen, value *output, unsigned char DollarAllowed, int RecursionDepth, int *status, char *errtext)
  {
-  int BracketLevel=0, offset=0;
+  int BracketLevel=0, outlen, offset=0;
 
   *status=-1;
   ppl_units_zero(output);
-  output->string = lt_malloc(LSTR_LENGTH);
+  outlen = 5 * (inlen+10); // Guess length of texified output to be no more than five times original length
+  if (outlen > LSTR_LENGTH) outlen = LSTR_LENGTH;
+  output->string = lt_malloc(outlen);
   if (output->string==NULL) { sprintf(errtext,"Out of memory."); *status=0; return; }
   output->string[0] = '\0';
   inlen--; // Make inlen point to last character
