@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: ppl_units.c 745 2010-05-03 18:06:33Z dcf21 $
+// $Id: ppl_units.c 883 2010-07-13 17:19:51Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -149,7 +149,7 @@ char *ppl_units_NumericDisplay(value *in, int N, int typeable, int NSigFigs)
 
 void ppl_units_FindOptimalNextUnit(value *in, unsigned char first, unit **best, double *pow)
  {
-  int i,j,k,score,found=0,BestScore;
+  int i,j,k,score,found=0,BestScore=0;
   double power;
 
   for (i=0; i<ppl_unit_pos; i++)
@@ -207,7 +207,7 @@ void ppl_units_PrefixFix(value *in, unit **UnitList, double *UnitPow, int *UnitP
  {
   int     i,j;
   double  NewValueReal, NewValueImag, PrefixBestVal, NewMagnitude, OldMagnitude;
-  int     PrefixBestPos, BestPrefix;
+  int     PrefixBestPos, BestPrefix=0;
 
   // Apply unit multipliers to the value we're going to display
   for (i=0; i<Nunits; i++)
@@ -287,9 +287,9 @@ char *ppl_units_GetUnitStr(const value *in, double *NumberOutReal, double *Numbe
   double        UnitPow [UNITS_MAX_BASEUNITS];
   int           UnitPref[UNITS_MAX_BASEUNITS];
   unsigned char UnitDisp[UNITS_MAX_BASEUNITS];
-  double        ExpMax;
+  double        ExpMax=0;
   int           pos=0, OutputPos=0;
-  int           i, j, k, l, found, first;
+  int           i, j=0, k, l, found, first;
   ListIterator *listiter;
   PreferredUnit *pu;
 
@@ -517,7 +517,7 @@ void ppl_units_StringEvaluate(char *in, value *out, int *end, int *errpos, char
 
       for (k=0; k<UNITS_MAX_BASEUNITS; k++) out->exponent[k] += ppl_unit_database[j].exponent[k] * power * powerneg;
       if (ppl_units_DblEqual(out->exponent[UNIT_TEMPERATURE], 0)) out->TempType = 0; // We've lost our temperature dependence
-      out->real *= multiplier * pow( ppl_unit_database[j].multiplier , power*powerneg );
+      out->real *= pow( multiplier * ppl_unit_database[j].multiplier , power*powerneg );
       power = 1.0;
       if      (in[i]=='*') { powerneg= 1.0; i++; }
       else if (in[i]=='/') { powerneg=-1.0; i++; }
