@@ -6,7 +6,7 @@
 // Copyright (C) 2006-2010 Dominic Ford <coders@pyxplot.org.uk>
 //               2008-2010 Ross Church
 //
-// $Id: ppl_userspace.c 763 2010-05-13 16:37:14Z dcf21 $
+// $Id: ppl_userspace.c 934 2010-08-26 00:39:29Z dcf21 $
 //
 // PyXPlot is free software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the Free Software
@@ -285,27 +285,45 @@ void ppl_UserSpace_FuncDestroy(FunctionDescriptor *in)
    }
   else if (in->FunctionType == PPL_USERSPACE_SPLINE)
    {
-    if (in->FunctionPtr!=NULL)
+    SplineDescriptor *d = (SplineDescriptor *)in->FunctionPtr;
+    if (d!=NULL)
      {
-      if (((SplineDescriptor *)in->FunctionPtr)->SplineObj   != NULL ) gsl_spline_free      (((SplineDescriptor *)in->FunctionPtr)->SplineObj  );
-      if (((SplineDescriptor *)in->FunctionPtr)->accelerator != NULL ) gsl_interp_accel_free(((SplineDescriptor *)in->FunctionPtr)->accelerator);
+      if (d->SplineType[1]!='t') // Not stepwise interpolation
+       {
+        if (d->SplineObj   != NULL) gsl_spline_free      (d->SplineObj  );
+        if (d->accelerator != NULL) gsl_interp_accel_free(d->accelerator);
+       }
+      else
+       {
+        if (d->SplineObj   != NULL) free(d->SplineObj);
+       }
+     }
+   }
+  else if ((in->FunctionType == PPL_USERSPACE_INTERP2D) || (in->FunctionType == PPL_USERSPACE_BMPDATA))
+   {
+    SplineDescriptor *d = (SplineDescriptor *)in->FunctionPtr;
+    if (d!=NULL)
+     {
+      if (d->SplineObj != NULL ) free(d->SplineObj);
      }
    }
   else if (in->FunctionType == PPL_USERSPACE_HISTOGRAM)
    {
-    if (in->FunctionPtr!=NULL)
+    HistogramDescriptor *h = (HistogramDescriptor *)in->FunctionPtr;
+    if (h!=NULL)
      {
-      if (((HistogramDescriptor *)in->FunctionPtr)->bins   != NULL ) free(((HistogramDescriptor *)in->FunctionPtr)->bins   );
-      if (((HistogramDescriptor *)in->FunctionPtr)->binvals!= NULL ) free(((HistogramDescriptor *)in->FunctionPtr)->binvals);
+      if (h->bins   != NULL ) free(h->bins   );
+      if (h->binvals!= NULL ) free(h->binvals);
      }
    }
   else if (in->FunctionType == PPL_USERSPACE_FFT)
    {
-    if (in->FunctionPtr!=NULL)
+    FFTDescriptor *f = (FFTDescriptor *)in->FunctionPtr;
+    if (f!=NULL)
      {
-      if (((FFTDescriptor *)in->FunctionPtr)->XSize    != NULL )      free(((FFTDescriptor *)in->FunctionPtr)->XSize   );
-      if (((FFTDescriptor *)in->FunctionPtr)->range    != NULL )      free(((FFTDescriptor *)in->FunctionPtr)->range   );
-      if (((FFTDescriptor *)in->FunctionPtr)->datagrid != NULL ) fftw_free(((FFTDescriptor *)in->FunctionPtr)->datagrid);
+      if (f->XSize    != NULL )      free(f->XSize   );
+      if (f->range    != NULL )      free(f->range   );
+      if (f->datagrid != NULL ) fftw_free(f->datagrid);
      }
    }
   else
@@ -331,7 +349,7 @@ void ppl_UserSpace_FuncDuplicate(FunctionDescriptor *in, int modified)
   if (in->FunctionType != PPL_USERSPACE_USERDEF) ppl_fatal(__FILE__,__LINE__,"Attempt to duplicate a function descriptor which is not a user-defined function.");
 
   Nargs=in->NumberArguments;
-  for (j=0,i=0;i<Nargs;i++) while (in->ArgList[j++]!='\0')
+  for (j=0,i=0;i<Nargs;i++) while (in->ArgList[j++]!='\0');
 
   if ((NewFuncPtr = (FunctionDescriptor *)lt_malloc_incontext(sizeof(FunctionDescriptor),0))==NULL) return;
        NewFuncPtr->FunctionType    = PPL_USERSPACE_USERDEF;
@@ -379,7 +397,7 @@ void ppl_GetQuotedString(char *in, char *out, int start, int *end, unsigned char
   FunctionDescriptor *FuncDefn = NULL;
   value ResultBuffer[ALGEBRA_MAXITEMS];       // A buffer of temporary numerical results
 
-  int    arg1i, arg2i; // Integers for passing to sprintf
+  int    arg1i=0, arg2i=0; // Integers for passing to sprintf
   value  arg1v, arg2v, argtemp; // Doubles (well, actually values with units) for passing to sprintf
   struct {double d,d2; int i,i2; char *s; unsigned int u,u2; value v; } argf; // We pass this lump of memory to sprintf
 
@@ -785,7 +803,7 @@ void ppl_EvaluateAlgebra(char *in, value *out, int start, int *end, unsigned cha
   int prev_start, prev_end, next_start, next_end, prev_bufno, next_bufno;
   unsigned char ci,cj;
   char dummy[DUMMYVAR_MAXLEN];
-  char ck, *integrand;
+  char ck, *integrand=NULL;
   int bufpos = 0;
   value  *VarData  = NULL;
   FunctionDescriptor *FuncDef = NULL;
@@ -924,6 +942,13 @@ void ppl_EvaluateAlgebra(char *in, value *out, int start, int *end, unsigned cha
         ppl_spline_evaluate(DictItem->key, (SplineDescriptor *)(((FunctionDescriptor *)DictItem->data)->FunctionPtr) , ResultBuffer+bufpos+2 , ResultBuffer+bufpos, &j, errtext);
         if (j>0) { *errpos = start+i; return; }
        }
+      else if ((FunctionType == PPL_USERSPACE_INTERP2D) || (FunctionType == PPL_USERSPACE_BMPDATA))
+       {
+        while (StatusRow[i]==3) i--; while ((i>0)&&(StatusRow[i]==8)) i--; if (StatusRow[i]!=8) i++; // Rewind back to beginning of f(x) text
+        j=0;
+        ppl_interp2d_evaluate(DictItem->key, (SplineDescriptor *)(((FunctionDescriptor *)DictItem->data)->FunctionPtr) , ResultBuffer+bufpos+2 , ResultBuffer+bufpos+3, (FunctionType == PPL_USERSPACE_BMPDATA), ResultBuffer+bufpos, &j, errtext);
+        if (j>0) { *errpos = start+i; return; }
+       }
       else if (FunctionType == PPL_USERSPACE_HISTOGRAM)
        {
         while (StatusRow[i]==3) i--; while ((i>0)&&(StatusRow[i]==8)) i--; if (StatusRow[i]!=8) i++; // Rewind back to beginning of f(x) text
@@ -1126,8 +1151,15 @@ void ppl_EvaluateAlgebra(char *in, value *out, int start, int *end, unsigned cha
   if (OpList[3]!=0) for (i=start,p=0;i<CalculatedEnd;i++,p++) if (StatusRow[p]==5)
    {
     FETCHNEXT(next_start, next_bufno, next_end);
-    ResultBuffer[next_bufno].real *= -1;
-    if (ResultBuffer[next_bufno].FlagComplex) ResultBuffer[next_bufno].imag *= -1;
+    if (MATCH_ONE('-'))
+     {
+      ResultBuffer[next_bufno].real *= -1;
+      if (ResultBuffer[next_bufno].FlagComplex) ResultBuffer[next_bufno].imag *= -1;
+     } else {
+      ResultBuffer[next_bufno].real        = !((ResultBuffer[next_bufno].real==0)&&(ResultBuffer[next_bufno].imag==0));
+      ResultBuffer[next_bufno].imag        = 0;
+      ResultBuffer[next_bufno].FlagComplex = 0;
+     }
     SETSTATUS(p, next_start, next_bufno);
     i = start + next_start;
     i--; p=i-start;
